@@ -11,12 +11,7 @@
 
 typedef enum {
     UNKNOWN = 0,
-    METADATA_INFO,
-    METADATA_ALARMS,
     HTTP_API_V2,
-    CHART_NEW,
-    CHART_DEL,
-    ALARM_STATE_UPDATE,
     REGISTER_NODE,
     NODE_STATE_UPDATE,
     CHART_DIMS_UPDATE,
@@ -24,22 +19,14 @@ typedef enum {
     CHART_RESET,
     RETENTION_UPDATED,
     UPDATE_NODE_INFO,
-    ALARM_LOG_HEALTH,
+    ALARM_PROVIDE_CHECKPOINT,
     ALARM_PROVIDE_CFG,
     ALARM_SNAPSHOT,
+    UPDATE_NODE_COLLECTORS,
+    PROTO_BIN_MESSAGE,
     ACLK_QUERY_TYPE_COUNT // always keep this as last
 } aclk_query_type_t;
 
-struct aclk_query_metadata {
-    RRDHOST *host;
-    int initial_on_connect;
-};
-
-struct aclk_query_chart_add_del {
-    RRDHOST *host;
-    char* chart_name;
-};
-
 struct aclk_query_http_api_v2 {
     char *payload;
     char *query;
@@ -73,12 +60,8 @@ struct aclk_query {
     // TODO maybe remove?
     int version;
     union {
-        struct aclk_query_metadata metadata_info;
-        struct aclk_query_metadata metadata_alarms;
         struct aclk_query_http_api_v2 http_api_v2;
-        struct aclk_query_chart_add_del chart_add_del;
         struct aclk_bin_payload bin_payload;
-        json_object *alarm_update;
     } data;
 };
 
