@@ -24,7 +24,17 @@
 #define NETDATA_CHART_PRIO_SYSTEM_PGPGIO                151
 #define NETDATA_CHART_PRIO_SYSTEM_RAM                   200
 #define NETDATA_CHART_PRIO_SYSTEM_SWAP                  201
+#define NETDATA_CHART_PRIO_SYSTEM_SWAP_CALLS            202
 #define NETDATA_CHART_PRIO_SYSTEM_SWAPIO                250
+#define NETDATA_CHART_PRIO_SYSTEM_ZSWAPIO               300
+#define NETDATA_CHART_PRIO_SYSTEM_ZSWAP_COMPRESS_RATIO  301
+#define NETDATA_CHART_PRIO_SYSTEM_ZSWAP_POOL_TOT_SIZE   302
+#define NETDATA_CHART_PRIO_SYSTEM_ZSWAP_STORED_PAGE     303
+#define NETDATA_CHART_PRIO_SYSTEM_ZSWAP_REJECTS         304
+#define NETDATA_CHART_PRIO_SYSTEM_ZSWAP_POOL_LIM_HIT    305
+#define NETDATA_CHART_PRIO_SYSTEM_ZSWAP_WRT_BACK_PAGES  306
+#define NETDATA_CHART_PRIO_SYSTEM_ZSWAP_SAME_FILL_PAGE  307
+#define NETDATA_CHART_PRIO_SYSTEM_ZSWAP_DUPP_ENTRY      308
 #define NETDATA_CHART_PRIO_SYSTEM_NET                   500
 #define NETDATA_CHART_PRIO_SYSTEM_IPV4                  500 // freebsd only
 #define NETDATA_CHART_PRIO_SYSTEM_IP                    501
@@ -80,9 +90,18 @@
 #define NETDATA_CHART_PRIO_MEM_KERNEL                 1100
 #define NETDATA_CHART_PRIO_MEM_SLAB                   1200
 #define NETDATA_CHART_PRIO_MEM_HUGEPAGES              1250
+#define NETDATA_CHART_PRIO_MEM_HUGEPAGES_FAULTS       1251
+#define NETDATA_CHART_PRIO_MEM_HUGEPAGES_FILE         1252
+#define NETDATA_CHART_PRIO_MEM_HUGEPAGES_ZERO         1253
+#define NETDATA_CHART_PRIO_MEM_HUGEPAGES_KHUGEPAGED   1254
+#define NETDATA_CHART_PRIO_MEM_HUGEPAGES_SPLITS       1255
+#define NETDATA_CHART_PRIO_MEM_HUGEPAGES_SWAPOUT      1256
+#define NETDATA_CHART_PRIO_MEM_HUGEPAGES_COMPACT      1257
 #define NETDATA_CHART_PRIO_MEM_KSM                    1300
 #define NETDATA_CHART_PRIO_MEM_KSM_SAVINGS            1301
 #define NETDATA_CHART_PRIO_MEM_KSM_RATIOS             1302
+#define NETDATA_CHART_PRIO_MEM_KSM_COW                1303
+#define NETDATA_CHART_PRIO_MEM_BALLOON                1350
 #define NETDATA_CHART_PRIO_MEM_NUMA                   1400
 #define NETDATA_CHART_PRIO_MEM_NUMA_NODES             1410
 #define NETDATA_CHART_PRIO_MEM_PAGEFRAG               1450
@@ -93,6 +112,7 @@
 #define NETDATA_CHART_PRIO_MEM_ZRAM_SAVINGS           1601
 #define NETDATA_CHART_PRIO_MEM_ZRAM_RATIO             1602
 #define NETDATA_CHART_PRIO_MEM_ZRAM_EFFICIENCY        1603
+#define NETDATA_CHART_PRIO_MEM_FRAGMENTATION          1700
 
 // Disks
 
@@ -182,6 +202,10 @@
 #define NETDATA_CHART_PRIO_BTRFS_DATA                 2401
 #define NETDATA_CHART_PRIO_BTRFS_METADATA             2402
 #define NETDATA_CHART_PRIO_BTRFS_SYSTEM               2403
+#define NETDATA_CHART_PRIO_BTRFS_COMMITS              2404
+#define NETDATA_CHART_PRIO_BTRFS_COMMITS_PERC_TIME    2405
+#define NETDATA_CHART_PRIO_BTRFS_COMMIT_TIMINGS       2406
+#define NETDATA_CHART_PRIO_BTRFS_ERRORS               2407
 
 // ZFS
 
