@@ -8,27 +8,122 @@ struct config fs_config = { .first_section = NULL,
     .index = { .avl_tree = { .root = NULL, .compar = appconfig_section_compare },
         .rwlock = AVL_LOCK_INITIALIZER } };
 
-static ebpf_local_maps_t fs_maps[] = {{.name = "tbl_ext4", .internal_input = NETDATA_KEY_CALLS_SYNC,
-                                       .user_input = 0, .type = NETDATA_EBPF_MAP_STATIC,
-                                       .map_fd = ND_EBPF_MAP_FD_NOT_INITIALIZED},
-                                      {.name = "tbl_xfs", .internal_input = NETDATA_KEY_CALLS_SYNC,
-                                       .user_input = 0, .type = NETDATA_EBPF_MAP_STATIC,
-                                       .map_fd = ND_EBPF_MAP_FD_NOT_INITIALIZED},
-                                      {.name = "tbl_nfs", .internal_input = NETDATA_KEY_CALLS_SYNC,
-                                       .user_input = 0, .type = NETDATA_EBPF_MAP_STATIC,
-                                       .map_fd = ND_EBPF_MAP_FD_NOT_INITIALIZED},
-                                      {.name = "tbl_zfs", .internal_input = NETDATA_KEY_CALLS_SYNC,
-                                       .user_input = 0, .type = NETDATA_EBPF_MAP_STATIC,
-                                       .map_fd = ND_EBPF_MAP_FD_NOT_INITIALIZED},
-                                      {.name = "tbl_btrfs", .internal_input = NETDATA_KEY_CALLS_SYNC,
-                                       .user_input = 0, .type = NETDATA_EBPF_MAP_STATIC,
-                                       .map_fd = ND_EBPF_MAP_FD_NOT_INITIALIZED},
-                                       {.name = "tbl_ext_addr", .internal_input = 1,
-                                       .user_input = 0, .type = NETDATA_EBPF_MAP_STATIC,
-                                       .map_fd = ND_EBPF_MAP_FD_NOT_INITIALIZED},
-                                      {.name = NULL, .internal_input = 0, .user_input = 0,
-                                       .type = NETDATA_EBPF_MAP_CONTROLLER,
-                                       .map_fd = ND_EBPF_MAP_FD_NOT_INITIALIZED}};
+ebpf_local_maps_t ext4_maps[] = {{.name = "tbl_ext4", .internal_input = NETDATA_KEY_CALLS_SYNC,
+                                  .user_input = 0, .type = NETDATA_EBPF_MAP_STATIC,
+                                  .map_fd = ND_EBPF_MAP_FD_NOT_INITIALIZED,
+#ifdef LIBBPF_MAJOR_VERSION
+                                  .map_type = BPF_MAP_TYPE_PERCPU_ARRAY
+#endif
+                                  },
+                                  {.name = "tmp_ext4", .internal_input = 4192, .user_input = 4192,
+                                   .type = NETDATA_EBPF_MAP_CONTROLLER,
+                                   .map_fd = ND_EBPF_MAP_FD_NOT_INITIALIZED,
+#ifdef LIBBPF_MAJOR_VERSION
+                                   .map_type = BPF_MAP_TYPE_PERCPU_HASH
+#endif
+                                   },
+                                   {.name = NULL, .internal_input = 0, .user_input = 0,
+                                    .type = NETDATA_EBPF_MAP_CONTROLLER,
+                                    .map_fd = ND_EBPF_MAP_FD_NOT_INITIALIZED,
+#ifdef LIBBPF_MAJOR_VERSION
+                                    .map_type = BPF_MAP_TYPE_PERCPU_HASH
+#endif
+                                    }};
+
+ebpf_local_maps_t xfs_maps[] = {{.name = "tbl_xfs", .internal_input = NETDATA_KEY_CALLS_SYNC,
+                                 .user_input = 0, .type = NETDATA_EBPF_MAP_STATIC,
+                                 .map_fd = ND_EBPF_MAP_FD_NOT_INITIALIZED,
+#ifdef LIBBPF_MAJOR_VERSION
+                                 .map_type = BPF_MAP_TYPE_PERCPU_ARRAY
+#endif
+                                 },
+                                {.name = "tmp_xfs", .internal_input = 4192, .user_input = 4192,
+                                 .type = NETDATA_EBPF_MAP_CONTROLLER,
+                                 .map_fd = ND_EBPF_MAP_FD_NOT_INITIALIZED,
+#ifdef LIBBPF_MAJOR_VERSION
+                                 .map_type = BPF_MAP_TYPE_PERCPU_HASH
+#endif
+                                 },
+                                 {.name = NULL, .internal_input = 0, .user_input = 0,
+                                  .type = NETDATA_EBPF_MAP_CONTROLLER,
+                                  .map_fd = ND_EBPF_MAP_FD_NOT_INITIALIZED,
+#ifdef LIBBPF_MAJOR_VERSION
+                                  .map_type = BPF_MAP_TYPE_PERCPU_HASH
+#endif
+                                 }};
+
+ebpf_local_maps_t nfs_maps[] = {{.name = "tbl_nfs", .internal_input = NETDATA_KEY_CALLS_SYNC,
+                                 .user_input = 0, .type = NETDATA_EBPF_MAP_STATIC,
+                                 .map_fd = ND_EBPF_MAP_FD_NOT_INITIALIZED,
+#ifdef LIBBPF_MAJOR_VERSION
+                                 .map_type = BPF_MAP_TYPE_PERCPU_ARRAY
+#endif
+                                 },
+                                {.name = "tmp_nfs", .internal_input = 4192, .user_input = 4192,
+                                 .type = NETDATA_EBPF_MAP_CONTROLLER,
+                                 .map_fd = ND_EBPF_MAP_FD_NOT_INITIALIZED,
+#ifdef LIBBPF_MAJOR_VERSION
+                                 .map_type = BPF_MAP_TYPE_PERCPU_HASH
+#endif
+                                },
+                                {.name = NULL, .internal_input = 0, .user_input = 0,
+                                 .type = NETDATA_EBPF_MAP_CONTROLLER,
+                                 .map_fd = ND_EBPF_MAP_FD_NOT_INITIALIZED,
+#ifdef LIBBPF_MAJOR_VERSION
+                                 .map_type = BPF_MAP_TYPE_PERCPU_HASH
+#endif
+                                 }};
+
+ebpf_local_maps_t zfs_maps[] = {{.name = "tbl_zfs", .internal_input = NETDATA_KEY_CALLS_SYNC,
+                                 .user_input = 0, .type = NETDATA_EBPF_MAP_STATIC,
+                                 .map_fd = ND_EBPF_MAP_FD_NOT_INITIALIZED,
+#ifdef LIBBPF_MAJOR_VERSION
+                                 .map_type = BPF_MAP_TYPE_PERCPU_ARRAY
+#endif
+                                },
+                                {.name = "tmp_zfs", .internal_input = 4192, .user_input = 4192,
+                                 .type = NETDATA_EBPF_MAP_CONTROLLER,
+                                 .map_fd = ND_EBPF_MAP_FD_NOT_INITIALIZED,
+#ifdef LIBBPF_MAJOR_VERSION
+                                 .map_type = BPF_MAP_TYPE_PERCPU_HASH
+#endif
+                                },
+                                {.name = NULL, .internal_input = 0, .user_input = 0,
+                                 .type = NETDATA_EBPF_MAP_CONTROLLER,
+                                 .map_fd = ND_EBPF_MAP_FD_NOT_INITIALIZED,
+#ifdef LIBBPF_MAJOR_VERSION
+                                 .map_type = BPF_MAP_TYPE_PERCPU_HASH
+#endif
+                                }};
+
+ebpf_local_maps_t btrfs_maps[] = {{.name = "tbl_btrfs", .internal_input = NETDATA_KEY_CALLS_SYNC,
+                                   .user_input = 0, .type = NETDATA_EBPF_MAP_STATIC,
+                                   .map_fd = ND_EBPF_MAP_FD_NOT_INITIALIZED,
+#ifdef LIBBPF_MAJOR_VERSION
+                                   .map_type = BPF_MAP_TYPE_PERCPU_ARRAY
+#endif
+                                },
+                                {.name = "tbl_ext_addr", .internal_input = 1, .user_input = 1,
+                                 .type = NETDATA_EBPF_MAP_CONTROLLER,
+                                 .map_fd = ND_EBPF_MAP_FD_NOT_INITIALIZED,
+#ifdef LIBBPF_MAJOR_VERSION
+                                 .map_type = BPF_MAP_TYPE_PERCPU_HASH
+#endif
+                                },
+                                {.name = "tmp_btrfs", .internal_input = 4192, .user_input = 4192,
+                                 .type = NETDATA_EBPF_MAP_CONTROLLER,
+                                 .map_fd = ND_EBPF_MAP_FD_NOT_INITIALIZED,
+#ifdef LIBBPF_MAJOR_VERSION
+                                 .map_type = BPF_MAP_TYPE_PERCPU_HASH
+#endif
+                                },
+                                {.name = NULL, .internal_input = 0, .user_input = 0,
+                                 .type = NETDATA_EBPF_MAP_CONTROLLER,
+                                 .map_fd = ND_EBPF_MAP_FD_NOT_INITIALIZED,
+#ifdef LIBBPF_MAJOR_VERSION
+                                 .map_type = BPF_MAP_TYPE_PERCPU_HASH
+#endif
+    }};
 
 static netdata_syscall_stat_t filesystem_aggregated_data[NETDATA_EBPF_HIST_MAX_BINS];
 static netdata_publish_syscall_t filesystem_publish_aggregated[NETDATA_EBPF_HIST_MAX_BINS];
@@ -92,7 +187,7 @@ static void ebpf_obsolete_fs_charts(int update_every)
 static void ebpf_create_fs_charts(int update_every)
 {
     static int order = NETDATA_CHART_PRIO_EBPF_FILESYSTEM_CHARTS;
-    char chart_name[64], title[256], family[64];
+    char chart_name[64], title[256], family[64], ctx[64];
     int i;
     uint32_t test = NETDATA_FILESYSTEM_FLAG_CHART_CREATED|NETDATA_FILESYSTEM_REMOVE_CHARTS;
     for (i = 0; localfs[i].filesystem; i++) {
@@ -110,7 +205,7 @@ static void ebpf_create_fs_charts(int update_every)
             ebpf_create_chart(NETDATA_FILESYSTEM_FAMILY, efp->hread.name,
                               title,
                               EBPF_COMMON_DIMENSION_CALL, family,
-                              NULL, NETDATA_EBPF_CHART_TYPE_STACKED, order, ebpf_create_global_dimension,
+                              "filesystem.read_latency", NETDATA_EBPF_CHART_TYPE_STACKED, order, ebpf_create_global_dimension,
                               filesystem_publish_aggregated, NETDATA_EBPF_HIST_MAX_BINS,
                               update_every, NETDATA_EBPF_MODULE_NAME_FILESYSTEM);
             order++;
@@ -123,7 +218,7 @@ static void ebpf_create_fs_charts(int update_every)
             ebpf_create_chart(NETDATA_FILESYSTEM_FAMILY, efp->hwrite.name,
                               title,
                               EBPF_COMMON_DIMENSION_CALL, family,
-                              NULL, NETDATA_EBPF_CHART_TYPE_STACKED, order, ebpf_create_global_dimension,
+                              "filesystem.write_latency", NETDATA_EBPF_CHART_TYPE_STACKED, order, ebpf_create_global_dimension,
                               filesystem_publish_aggregated, NETDATA_EBPF_HIST_MAX_BINS,
                               update_every, NETDATA_EBPF_MODULE_NAME_FILESYSTEM);
             order++;
@@ -136,7 +231,7 @@ static void ebpf_create_fs_charts(int update_every)
             ebpf_create_chart(NETDATA_FILESYSTEM_FAMILY, efp->hopen.name,
                               title,
                               EBPF_COMMON_DIMENSION_CALL, family,
-                              NULL, NETDATA_EBPF_CHART_TYPE_STACKED, order, ebpf_create_global_dimension,
+                              "filesystem.open_latency", NETDATA_EBPF_CHART_TYPE_STACKED, order, ebpf_create_global_dimension,
                               filesystem_publish_aggregated, NETDATA_EBPF_HIST_MAX_BINS,
                               update_every, NETDATA_EBPF_MODULE_NAME_FILESYSTEM);
             order++;
@@ -144,12 +239,13 @@ static void ebpf_create_fs_charts(int update_every)
             char *type = (efp->flags & NETDATA_FILESYSTEM_ATTR_CHARTS) ? "attribute" : "sync";
             snprintfz(title, 255, "%s latency for each %s request.", efp->filesystem, type);
             snprintfz(chart_name, 63, "%s_%s_latency", efp->filesystem, type);
+            snprintfz(ctx, 63, "filesystem.%s_latency", type);
             efp->hadditional.name = strdupz(chart_name);
             efp->hadditional.title = strdupz(title);
             efp->hadditional.order = order;
             ebpf_create_chart(NETDATA_FILESYSTEM_FAMILY, efp->hadditional.name, title,
                               EBPF_COMMON_DIMENSION_CALL, family,
-                              NULL, NETDATA_EBPF_CHART_TYPE_STACKED, order, ebpf_create_global_dimension,
+                              ctx, NETDATA_EBPF_CHART_TYPE_STACKED, order, ebpf_create_global_dimension,
                               filesystem_publish_aggregated, NETDATA_EBPF_HIST_MAX_BINS,
                               update_every, NETDATA_EBPF_MODULE_NAME_FILESYSTEM);
             order++;
@@ -175,23 +271,32 @@ int ebpf_filesystem_initialize_ebpf_data(ebpf_module_t *em)
         if (!efp->probe_links && efp->flags & NETDATA_FILESYSTEM_LOAD_EBPF_PROGRAM) {
             em->thread_name = efp->filesystem;
             em->kernels = efp->kernels;
+            em->maps = efp->fs_maps;
+#ifdef LIBBPF_MAJOR_VERSION
+            ebpf_define_map_type(em->maps, em->maps_per_core, running_on_kernel);
+#endif
             efp->probe_links = ebpf_load_program(ebpf_plugin_dir, em, running_on_kernel, isrh, &efp->objects);
             if (!efp->probe_links) {
                 em->thread_name = saved_name;
                 em->kernels = kernels;
+                em->maps = NULL;
                 return -1;
             }
             efp->flags |= NETDATA_FILESYSTEM_FLAG_HAS_PARTITION;
+            pthread_mutex_lock(&lock);
+            ebpf_update_kernel_memory(&plugin_statistics, efp->fs_maps, EBPF_ACTION_STAT_ADD);
+            pthread_mutex_unlock(&lock);
 
             // Nedeed for filesystems like btrfs
             if ((efp->flags & NETDATA_FILESYSTEM_FILL_ADDRESS_TABLE) && (efp->addresses.function)) {
-                ebpf_load_addresses(&efp->addresses, fs_maps[i + 1].map_fd);
+                ebpf_load_addresses(&efp->addresses, efp->fs_maps[NETDATA_ADDR_FS_TABLE].map_fd);
             }
         }
         efp->flags &= ~NETDATA_FILESYSTEM_LOAD_EBPF_PROGRAM;
     }
     em->thread_name = saved_name;
     em->kernels = kernels;
+    em->maps = NULL;
 
     if (!dimensions) {
         dimensions = ebpf_fill_histogram_dimension(NETDATA_EBPF_HIST_MAX_BINS);
@@ -326,18 +431,16 @@ void ebpf_filesystem_cleanup_ebpf_data()
 static void ebpf_filesystem_free(ebpf_module_t *em)
 {
     pthread_mutex_lock(&ebpf_exit_cleanup);
-    em->thread->enabled = NETDATA_THREAD_EBPF_STOPPING;
+    em->enabled = NETDATA_THREAD_EBPF_STOPPING;
     pthread_mutex_unlock(&ebpf_exit_cleanup);
 
-    ebpf_cleanup_publish_syscall(filesystem_publish_aggregated);
-
     ebpf_filesystem_cleanup_ebpf_data();
     if (dimensions)
         ebpf_histogram_dimension_cleanup(dimensions, NETDATA_EBPF_HIST_MAX_BINS);
     freez(filesystem_hash_values);
 
     pthread_mutex_lock(&ebpf_exit_cleanup);
-    em->thread->enabled = NETDATA_THREAD_EBPF_STOPPED;
+    em->enabled = NETDATA_THREAD_EBPF_STOPPED;
     pthread_mutex_unlock(&ebpf_exit_cleanup);
 }
 
@@ -392,11 +495,13 @@ static inline netdata_ebpf_histogram_t *select_hist(ebpf_filesystem_partitions_t
 /**
  * Read hard disk table
  *
- * @param table index for the hash table
+ * @param efp           structure with filesystem monitored
+ * @param fd            file descriptor to get data.
+ * @param maps_per_core do I need to read all cores?
  *
  * Read the table with number of calls for all functions
  */
-static void read_filesystem_table(ebpf_filesystem_partitions_t *efp, int fd)
+static void read_filesystem_table(ebpf_filesystem_partitions_t *efp, int fd, int maps_per_core)
 {
     netdata_idx_t *values = filesystem_hash_values;
     uint32_t key;
@@ -414,7 +519,7 @@ static void read_filesystem_table(ebpf_filesystem_partitions_t *efp, int fd)
 
         uint64_t total = 0;
         int i;
-        int end = ebpf_nprocs;
+        int end = (maps_per_core) ? ebpf_nprocs : 1;
         for (i = 0; i < end; i++) {
             total += values[i];
         }
@@ -428,17 +533,17 @@ static void read_filesystem_table(ebpf_filesystem_partitions_t *efp, int fd)
 /**
  * Read hard disk table
  *
- * @param table index for the hash table
- *
  * Read the table with number of calls for all functions
+ *
+ * @param maps_per_core do I need to read all cores?
  */
-static void read_filesystem_tables()
+static void read_filesystem_tables(int maps_per_core)
 {
     int i;
     for (i = 0; localfs[i].filesystem; i++) {
         ebpf_filesystem_partitions_t *efp = &localfs[i];
         if (efp->flags & NETDATA_FILESYSTEM_FLAG_HAS_PARTITION) {
-            read_filesystem_table(efp, fs_maps[i].map_fd);
+            read_filesystem_table(efp, efp->fs_maps[NETDATA_MAIN_FS_TABLE].map_fd, maps_per_core);
         }
     }
 }
@@ -462,7 +567,7 @@ void ebpf_filesystem_read_hash(ebpf_module_t *em)
     if (em->optional)
         return;
 
-    read_filesystem_tables();
+    read_filesystem_tables(em->maps_per_core);
 }
 
 /**
@@ -543,6 +648,21 @@ static void ebpf_update_filesystem()
     }
 }
 
+/**
+ * Set maps
+ *
+ * When thread is initialized the variable fs_maps is set as null,
+ * this function fills the variable before to use.
+ */
+static void ebpf_set_maps()
+{
+    localfs[NETDATA_FS_LOCALFS_EXT4].fs_maps = ext4_maps;
+    localfs[NETDATA_FS_LOCALFS_XFS].fs_maps = xfs_maps;
+    localfs[NETDATA_FS_LOCALFS_NFS].fs_maps = nfs_maps;
+    localfs[NETDATA_FS_LOCALFS_ZFS].fs_maps = zfs_maps;
+    localfs[NETDATA_FS_LOCALFS_BTRFS].fs_maps = btrfs_maps;
+}
+
 /**
  * Filesystem thread
  *
@@ -557,7 +677,7 @@ void *ebpf_filesystem_thread(void *ptr)
     netdata_thread_cleanup_push(ebpf_filesystem_exit, ptr);
 
     ebpf_module_t *em = (ebpf_module_t *)ptr;
-    em->maps = fs_maps;
+    ebpf_set_maps();
     ebpf_update_filesystem();
 
     // Initialize optional as zero, to identify when there are not partitions to monitor
@@ -567,7 +687,6 @@ void *ebpf_filesystem_thread(void *ptr)
         if (em->optional)
             info("Netdata cannot monitor the filesystems used on this host.");
 
-        em->thread->enabled = NETDATA_THREAD_EBPF_STOPPED;
         goto endfilesystem;
     }
 
