@@ -17,7 +17,6 @@ typedef uint8_t oomkill_ebpf_val_t;
  *****************************************************************/
 
 #define NETDATA_EBPF_MODULE_NAME_OOMKILL "oomkill"
-#define NETDATA_OOMKILL_SLEEP_MS 650000ULL
 #define NETDATA_OOMKILL_CONFIG_FILE "oomkill.conf"
 
 #define NETDATA_OOMKILL_CHART "oomkills"
@@ -26,7 +25,7 @@ typedef uint8_t oomkill_ebpf_val_t;
 #define NETDATA_CGROUP_OOMKILLS_CONTEXT "cgroup.oomkills"
 
 extern struct config oomkill_config;
-extern void *ebpf_oomkill_thread(void *ptr);
-extern void ebpf_oomkill_create_apps_charts(struct ebpf_module *em, void *ptr);
+void *ebpf_oomkill_thread(void *ptr);
+void ebpf_oomkill_create_apps_charts(struct ebpf_module *em, void *ptr);
 
 #endif /* NETDATA_EBPF_OOMKILL_H */
