@@ -56,6 +56,13 @@ enum netdata_ebpf_load_mode_stats{
     NETDATA_EBPF_LOAD_STAT_END
 };
 
+enum netdata_ebpf_thread_per_core{
+    NETDATA_EBPF_THREAD_PER_CORE,
+    NETDATA_EBPF_THREAD_UNIQUE,
+
+    NETDATA_EBPF_PER_CORE_END
+};
+
 // Index from kernel
 typedef enum ebpf_process_index {
     NETDATA_KEY_CALLS_DO_EXIT,
@@ -85,17 +92,6 @@ typedef enum netdata_publish_process {
     NETDATA_KEY_PUBLISH_PROCESS_END
 } netdata_publish_process_t;
 
-typedef struct ebpf_process_publish_apps {
-    // Number of calls during the last read
-    uint64_t call_do_exit;
-    uint64_t call_release_task;
-    uint64_t create_process;
-    uint64_t create_thread;
-
-    // Number of errors during the last read
-    uint64_t task_err;
-} ebpf_process_publish_apps_t;
-
 enum ebpf_process_tables {
     NETDATA_PROCESS_PID_TABLE,
     NETDATA_PROCESS_GLOBAL_TABLE,
