@@ -1,6 +1,8 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
 
-#include "../../libnetdata/libnetdata.h"
+#include "libnetdata/libnetdata.h"
+#include "libnetdata/required_dummies.h"
+
 #include <linux/netfilter/nfnetlink_conntrack.h>
 #include <libmnl/libmnl.h>
 #include <libnetfilter_acct/libnetfilter_acct.h>
@@ -22,34 +24,6 @@ static inline size_t mnl_buffer_size() {
     return (size_t)s;
 }
 
-// callback required by fatal()
-void netdata_cleanup_and_exit(int ret) {
-    exit(ret);
-}
-
-void send_statistics( const char *action, const char *action_result, const char *action_data) {
-    (void) action;
-    (void) action_result;
-    (void) action_data;
-    return;
-}
-
-// callbacks required by popen()
-void signals_block(void) {};
-void signals_unblock(void) {};
-void signals_reset(void) {};
-
-// callback required by eval()
-int health_variable_lookup(const char *variable, uint32_t hash, struct rrdcalc *rc, calculated_number *result) {
-    (void)variable;
-    (void)hash;
-    (void)rc;
-    (void)result;
-    return 0;
-};
-
-// required by get_system_cpus()
-char *netdata_configured_host_prefix = "";
 // variables
 static int debug = 0;
 static int netdata_update_every = 1;
@@ -118,14 +92,14 @@ static int nfstat_init(int update_every) {
 
     nfstat_root.mnl  = mnl_socket_open(NETLINK_NETFILTER);
     if(!nfstat_root.mnl) {
-        error("NFSTAT: mnl_socket_open() failed");
+        collector_error("NFSTAT: mnl_socket_open() failed");
         return 1;
     }
 
     nfstat_root.seq = (unsigned int)now_realtime_sec() - 1;
 
     if(mnl_socket_bind(nfstat_root.mnl, 0, MNL_SOCKET_AUTOPID) < 0) {
-        error("NFSTAT: mnl_socket_bind() failed");
+        collector_error("NFSTAT: mnl_socket_bind() failed");
         return 1;
     }
     nfstat_root.portid = mnl_socket_get_portid(nfstat_root.mnl);
@@ -158,7 +132,7 @@ static int nfct_stats_attr_cb(const struct nlattr *attr, void *data) {
         return MNL_CB_OK;
 
     if (mnl_attr_validate(attr, MNL_TYPE_U32) < 0) {
-        error("NFSTAT: mnl_attr_validate() failed");
+        collector_error("NFSTAT: mnl_attr_validate() failed");
         return MNL_CB_ERROR;
     }
 
@@ -199,7 +173,7 @@ static int nfstat_collect_conntrack() {
 
     // send the request
     if(mnl_socket_sendto(nfstat_root.mnl, nfstat_root.nlh, nfstat_root.nlh->nlmsg_len) < 0) {
-        error("NFSTAT: mnl_socket_sendto() failed");
+        collector_error("NFSTAT: mnl_socket_sendto() failed");
         return 1;
     }
 
@@ -219,7 +193,7 @@ static int nfstat_collect_conntrack() {
 
     // verify we run without issues
     if (ret == -1) {
-        error("NFSTAT: error communicating with kernel. This plugin can only work when netdata runs as root.");
+        collector_error("NFSTAT: error communicating with kernel. This plugin can only work when netdata runs as root.");
         return 1;
     }
 
@@ -235,7 +209,7 @@ static int nfexp_stats_attr_cb(const struct nlattr *attr, void *data)
         return MNL_CB_OK;
 
     if (mnl_attr_validate(attr, MNL_TYPE_U32) < 0) {
-        error("NFSTAT EXP: mnl_attr_validate() failed");
+        collector_error("NFSTAT EXP: mnl_attr_validate() failed");
         return MNL_CB_ERROR;
     }
 
@@ -271,7 +245,7 @@ static int nfstat_collect_conntrack_expectations() {
 
     // send the request
     if(mnl_socket_sendto(nfstat_root.mnl, nfstat_root.nlh, nfstat_root.nlh->nlmsg_len) < 0) {
-        error("NFSTAT: mnl_socket_sendto() failed");
+        collector_error("NFSTAT: mnl_socket_sendto() failed");
         return 1;
     }
 
@@ -291,7 +265,7 @@ static int nfstat_collect_conntrack_expectations() {
 
     // verify we run without issues
     if (ret == -1) {
-        error("NFSTAT: error communicating with kernel. This plugin can only work when netdata runs as root.");
+        collector_error("NFSTAT: error communicating with kernel. This plugin can only work when netdata runs as root.");
         return 1;
     }
 
@@ -587,7 +561,7 @@ static int nfacct_init(int update_every) {
 
     nfacct_root.nfacct_buffer = nfacct_alloc();
     if(!nfacct_root.nfacct_buffer) {
-        error("nfacct.plugin: nfacct_alloc() failed.");
+        collector_error("nfacct.plugin: nfacct_alloc() failed.");
         return 0;
     }
 
@@ -595,12 +569,12 @@ static int nfacct_init(int update_every) {
 
     nfacct_root.mnl  = mnl_socket_open(NETLINK_NETFILTER);
     if(!nfacct_root.mnl) {
-        error("nfacct.plugin: mnl_socket_open() failed");
+        collector_error("nfacct.plugin: mnl_socket_open() failed");
         return 1;
     }
 
     if(mnl_socket_bind(nfacct_root.mnl, 0, MNL_SOCKET_AUTOPID) < 0) {
-        error("nfacct.plugin: mnl_socket_bind() failed");
+        collector_error("nfacct.plugin: mnl_socket_bind() failed");
         return 1;
     }
     nfacct_root.portid = mnl_socket_get_portid(nfacct_root.mnl);
@@ -612,7 +586,7 @@ static int nfacct_callback(const struct nlmsghdr *nlh, void *data) {
     (void)data;
 
     if(nfacct_nlmsg_parse_payload(nlh, nfacct_root.nfacct_buffer) < 0) {
-        error("NFACCT: nfacct_nlmsg_parse_payload() failed.");
+        collector_error("NFACCT: nfacct_nlmsg_parse_payload() failed.");
         return MNL_CB_OK;
     }
 
@@ -638,13 +612,13 @@ static int nfacct_collect() {
     nfacct_root.seq++;
     nfacct_root.nlh = nfacct_nlmsg_build_hdr(nfacct_root.buf, NFNL_MSG_ACCT_GET, NLM_F_DUMP, (uint32_t)nfacct_root.seq);
     if(!nfacct_root.nlh) {
-        error("NFACCT: nfacct_nlmsg_build_hdr() failed");
+        collector_error("NFACCT: nfacct_nlmsg_build_hdr() failed");
         return 1;
     }
 
     // send the request
     if(mnl_socket_sendto(nfacct_root.mnl, nfacct_root.nlh, nfacct_root.nlh->nlmsg_len) < 0) {
-        error("NFACCT: mnl_socket_sendto() failed");
+        collector_error("NFACCT: mnl_socket_sendto() failed");
         return 1;
     }
 
@@ -664,7 +638,7 @@ static int nfacct_collect() {
 
     // verify we run without issues
     if (ret == -1) {
-        error("NFACCT: error communicating with kernel. This plugin can only work when netdata runs as root.");
+        collector_error("NFACCT: error communicating with kernel. This plugin can only work when netdata runs as root.");
         return 1;
     }
 
@@ -690,7 +664,11 @@ static void nfacct_send_metrics() {
         if(likely(d->updated)) {
             if(unlikely(!d->packets_dimension_added)) {
                 d->packets_dimension_added = 1;
-                printf("CHART netfilter.nfacct_packets '' 'Netfilter Accounting Packets' 'packets/s'\n");
+                printf(
+                    "CHART netfilter.nfacct_packets '' 'Netfilter Accounting Packets' 'packets/s' 'nfacct' '' stacked %d %d %s\n",
+                    NETDATA_CHART_PRIO_NETFILTER_PACKETS,
+                    nfacct_root.update_every,
+                    PLUGIN_NFACCT_NAME);
                 printf("DIMENSION %s '' incremental 1 %d\n", d->name, nfacct_root.update_every);
             }
         }
@@ -721,7 +699,11 @@ static void nfacct_send_metrics() {
         if(likely(d->updated)) {
             if(unlikely(!d->bytes_dimension_added)) {
                 d->bytes_dimension_added = 1;
-                printf("CHART netfilter.nfacct_bytes '' 'Netfilter Accounting Bandwidth' 'kilobytes/s'\n");
+                printf(
+                    "CHART netfilter.nfacct_bytes '' 'Netfilter Accounting Bandwidth' 'kilobytes/s' 'nfacct' '' stacked %d %d %s\n",
+                    NETDATA_CHART_PRIO_NETFILTER_BYTES,
+                    nfacct_root.update_every,
+                    PLUGIN_NFACCT_NAME);
                 printf("DIMENSION %s '' incremental 1 %d\n", d->name, 1000 * nfacct_root.update_every);
             }
         }
@@ -758,11 +740,13 @@ void nfacct_signals()
 
     for (i = 0; signals[i]; i++) {
         if(sigaction(signals[i], &sa, NULL) == -1)
-            error("Cannot add the handler to signal %d", signals[i]);
+            collector_error("Cannot add the handler to signal %d", signals[i]);
     }
 }
 
 int main(int argc, char **argv) {
+    stderror = stderr;
+    clocks_init();
 
     // ------------------------------------------------------------------------
     // initialization of netdata plugin
@@ -830,7 +814,7 @@ int main(int argc, char **argv) {
             exit(1);
         }
 
-        error("nfacct.plugin: ignoring parameter '%s'", argv[i]);
+        collector_error("nfacct.plugin: ignoring parameter '%s'", argv[i]);
     }
 
     nfacct_signals();
@@ -840,7 +824,7 @@ int main(int argc, char **argv) {
     if(freq >= netdata_update_every)
         netdata_update_every = freq;
     else if(freq)
-        error("update frequency %d seconds is too small for NFACCT. Using %d.", freq, netdata_update_every);
+        collector_error("update frequency %d seconds is too small for NFACCT. Using %d.", freq, netdata_update_every);
 
     if (debug)
         fprintf(stderr, "nfacct.plugin: calling nfacct_init()\n");
@@ -899,5 +883,5 @@ int main(int argc, char **argv) {
         if(now_monotonic_sec() - started_t > 14400) break;
     }
 
-    info("NFACCT process exiting");
+    collector_info("NFACCT process exiting");
 }
