@@ -20,10 +20,10 @@
 #endif
 #endif
 
-#ifdef ENABLE_NEW_CLOUD_PROTOCOL
-#define NEW_CLOUD_PROTO 1
+#ifdef ENABLE_HTTPD
+#define FEAT_HTTPD 1
 #else
-#define NEW_CLOUD_PROTO 0
+#define FEAT_HTTPD 0
 #endif
 
 #ifdef ENABLE_DBENGINE
@@ -95,12 +95,6 @@
 #define FEAT_LIBCAP 0
 #endif
 
-#ifdef NETDATA_WITH_ZLIB
-#define FEAT_ZLIB 1
-#else
-#define FEAT_ZLIB 0
-#endif
-
 #ifdef STORAGE_WITH_MATH
 #define FEAT_LIBM 1
 #else
@@ -121,6 +115,12 @@
 #define FEAT_APPS_PLUGIN 0
 #endif
 
+#ifdef ENABLE_DEBUGFS_PLUGIN
+#define FEAT_DEBUGFS_PLUGIN 1
+#else
+#define FEAT_DEBUGFS_PLUGIN 0
+#endif
+
 #ifdef HAVE_FREEIPMI
 #define FEAT_IPMI 1
 #else
@@ -203,6 +203,11 @@
 
 #define FEAT_YES_NO(x) ((x) ? "YES" : "NO")
 
+#ifdef NETDATA_TRACE_ALLOCATIONS
+#define FEAT_TRACE_ALLOC 1
+#else
+#define FEAT_TRACE_ALLOC 0
+#endif
 
 char *get_value_from_key(char *buffer, char *key) {
     char *s = NULL, *t = NULL;
@@ -272,12 +277,11 @@ void print_build_info(void) {
     printf("    dbengine:                   %s\n", FEAT_YES_NO(FEAT_DBENGINE));
     printf("    Native HTTPS:               %s\n", FEAT_YES_NO(FEAT_NATIVE_HTTPS));
     printf("    Netdata Cloud:              %s %s\n", FEAT_YES_NO(FEAT_CLOUD), FEAT_CLOUD_MSG);
-    printf("    ACLK Next Generation:       %s\n", FEAT_YES_NO(FEAT_CLOUD));
-    printf("    ACLK-NG New Cloud Protocol: %s\n", FEAT_YES_NO(NEW_CLOUD_PROTO));
-    printf("    ACLK Legacy:                %s\n", FEAT_YES_NO(0));
+    printf("    ACLK:                       %s\n", FEAT_YES_NO(FEAT_CLOUD));
     printf("    TLS Host Verification:      %s\n", FEAT_YES_NO(FEAT_TLS_HOST_VERIFY));
     printf("    Machine Learning:           %s\n", FEAT_YES_NO(FEAT_ML));
     printf("    Stream Compression:         %s\n", FEAT_YES_NO(FEAT_STREAM_COMPRESSION));
+    printf("    HTTPD (h2o):                %s\n", FEAT_YES_NO(FEAT_HTTPD));
 
     printf("Libraries:\n");
     printf("    protobuf:                %s%s\n", FEAT_YES_NO(FEAT_PROTOBUF), FEAT_PROTOBUF_BUNDLED);
@@ -287,12 +291,13 @@ void print_build_info(void) {
     printf("    libcrypto:               %s\n", FEAT_YES_NO(FEAT_CRYPTO));
     printf("    libm:                    %s\n", FEAT_YES_NO(FEAT_LIBM));
     printf("    tcalloc:                 %s\n", FEAT_YES_NO(FEAT_TCMALLOC));
-    printf("    zlib:                    %s\n", FEAT_YES_NO(FEAT_ZLIB));
+    printf("    zlib:                    %s\n", FEAT_YES_NO(1));
 
     printf("Plugins:\n");
     printf("    apps:                    %s\n", FEAT_YES_NO(FEAT_APPS_PLUGIN));
     printf("    cgroup Network Tracking: %s\n", FEAT_YES_NO(FEAT_CGROUP_NET));
     printf("    CUPS:                    %s\n", FEAT_YES_NO(FEAT_CUPS));
+    printf("    debugfs:                 %s\n", FEAT_YES_NO(FEAT_DEBUGFS_PLUGIN));
     printf("    EBPF:                    %s\n", FEAT_YES_NO(FEAT_EBPF));
     printf("    IPMI:                    %s\n", FEAT_YES_NO(FEAT_IPMI));
     printf("    NFACCT:                  %s\n", FEAT_YES_NO(FEAT_NFACCT));
@@ -306,6 +311,9 @@ void print_build_info(void) {
     printf("    GCP PubSub:              %s\n", FEAT_YES_NO(FEAT_PUBSUB));
     printf("    MongoDB:                 %s\n", FEAT_YES_NO(FEAT_MONGO));
     printf("    Prometheus Remote Write: %s\n", FEAT_YES_NO(FEAT_REMOTE_WRITE));
+
+    printf("Debug/Developer Features:\n");
+    printf("    Trace Allocations:       %s\n", FEAT_YES_NO(FEAT_TRACE_ALLOC));
 };
 
 #define FEAT_JSON_BOOL(x) ((x) ? "true" : "false")
@@ -324,13 +332,12 @@ void print_build_info_json(void) {
 #else
     printf("    \"cloud-disabled\": false,\n");
 #endif
-    printf("    \"aclk-ng\": %s,\n", FEAT_JSON_BOOL(FEAT_CLOUD));
-    printf("    \"aclk-ng-new-cloud-proto\": %s,\n", FEAT_JSON_BOOL(NEW_CLOUD_PROTO));
-    printf("    \"aclk-legacy\": %s,\n", FEAT_JSON_BOOL(0));
+    printf("    \"aclk\": %s,\n", FEAT_JSON_BOOL(FEAT_CLOUD));
 
     printf("    \"tls-host-verify\": %s,\n",   FEAT_JSON_BOOL(FEAT_TLS_HOST_VERIFY));
     printf("    \"machine-learning\": %s\n",   FEAT_JSON_BOOL(FEAT_ML));
-    printf("    \"stream-compression\": %s\n",   FEAT_JSON_BOOL(FEAT_STREAM_COMPRESSION));
+    printf("    \"stream-compression\": %s\n", FEAT_JSON_BOOL(FEAT_STREAM_COMPRESSION));
+    printf("    \"httpd-h2o\": %s\n",          FEAT_JSON_BOOL(FEAT_HTTPD));
     printf("  },\n");
 
     printf("  \"libs\": {\n");
@@ -342,13 +349,14 @@ void print_build_info_json(void) {
     printf("    \"libcrypto\": %s,\n",        FEAT_JSON_BOOL(FEAT_CRYPTO));
     printf("    \"libm\": %s,\n",             FEAT_JSON_BOOL(FEAT_LIBM));
     printf("    \"tcmalloc\": %s,\n",         FEAT_JSON_BOOL(FEAT_TCMALLOC));
-    printf("    \"zlib\": %s\n",              FEAT_JSON_BOOL(FEAT_ZLIB));
+    printf("    \"zlib\": %s\n",              FEAT_JSON_BOOL(1));
     printf("  },\n");
 
     printf("  \"plugins\": {\n");
     printf("    \"apps\": %s,\n",             FEAT_JSON_BOOL(FEAT_APPS_PLUGIN));
     printf("    \"cgroup-net\": %s,\n",       FEAT_JSON_BOOL(FEAT_CGROUP_NET));
     printf("    \"cups\": %s,\n",             FEAT_JSON_BOOL(FEAT_CUPS));
+    printf("    \"debugfs\": %s,\n",          FEAT_JSON_BOOL(FEAT_DEBUGFS_PLUGIN));
     printf("    \"ebpf\": %s,\n",             FEAT_JSON_BOOL(FEAT_EBPF));
     printf("    \"ipmi\": %s,\n",             FEAT_JSON_BOOL(FEAT_IPMI));
     printf("    \"nfacct\": %s,\n",           FEAT_JSON_BOOL(FEAT_NFACCT));
@@ -364,6 +372,8 @@ void print_build_info_json(void) {
     printf("    \"mongodb\": %s,\n",          FEAT_JSON_BOOL(FEAT_MONGO));
     printf("    \"prom-remote-write\": %s\n", FEAT_JSON_BOOL(FEAT_REMOTE_WRITE));
     printf("  }\n");
+    printf("  \"debug-n-devel\": {\n");
+    printf("    \"trace-allocations\": %s\n  }\n",FEAT_JSON_BOOL(FEAT_TRACE_ALLOC));
     printf("}\n");
 };
 
@@ -383,10 +393,7 @@ void analytics_build_info(BUFFER *b) {
     add_to_bi(b, "Native HTTPS");
 #endif
 #ifdef ENABLE_ACLK
-    add_to_bi(b, "Netdata Cloud|ACLK Next Generation");
-#endif
-#ifdef ENABLE_NEW_CLOUD_PROTOCOL
-    add_to_bi(b, "New Cloud Protocol Support");
+    add_to_bi(b, "Netdata Cloud");
 #endif
 #if (FEAT_TLS_HOST_VERIFY!=0)
     add_to_bi(b, "TLS Host Verification");
@@ -420,13 +427,14 @@ void analytics_build_info(BUFFER *b) {
 #ifdef ENABLE_TCMALLOC
     add_to_bi(b, "tcalloc");
 #endif
-#ifdef NETDATA_WITH_ZLIB
     add_to_bi(b, "zlib");
-#endif
 
 #ifdef ENABLE_APPS_PLUGIN
     add_to_bi(b, "apps");
 #endif
+#ifdef ENABLE_DEBUGFS_PLUGIN
+    add_to_bi(b, "debugfs");
+#endif
 #ifdef HAVE_SETNS
     add_to_bi(b, "cgroup Network Tracking");
 #endif
@@ -467,4 +475,7 @@ void analytics_build_info(BUFFER *b) {
 #ifdef ENABLE_PROMETHEUS_REMOTE_WRITE
     add_to_bi(b, "Prometheus Remote Write");
 #endif
+#ifdef NETDATA_TRACE_ALLOCATIONS
+    add_to_bi(b, "DebugTraceAlloc");
+#endif
 }
