@@ -117,7 +117,6 @@ void sql_close_context_database(void)
     rc = sqlite3_close_v2(db_context_meta);
     if (unlikely(rc != SQLITE_OK))
         error_report("Error %d while closing the context SQLite database, %s", rc, sqlite3_errstr(rc));
-    return;
 }
 
 //
@@ -243,8 +242,6 @@ failed:
     rc = sqlite3_reset(res);
     if (rc != SQLITE_OK)
         error_report("Failed to reset statement that fetches chart label data, rc = %d", rc);
-
-    return;
 }
 
 // CONTEXT LIST
@@ -372,9 +369,9 @@ int ctx_store_context(uuid_t *host_uuid, VERSIONED_CONTEXT_DATA *context_data)
         goto skip_store;
     }
 
-    rc = sqlite3_bind_int(res, 10, (time_t) context_data->deleted);
+    rc = sqlite3_bind_int(res, 10, context_data->deleted);
     if (unlikely(rc != SQLITE_OK)) {
-        error_report("Failed to bind last_time_t to store context details");
+        error_report("Failed to bind deleted flag to store context details");
         goto skip_store;
     }
 
@@ -431,11 +428,11 @@ int ctx_delete_context(uuid_t *host_uuid, VERSIONED_CONTEXT_DATA *context_data)
     if (rc_stored != SQLITE_DONE)
         error_report("Failed to delete context %s, rc = %d", context_data->id, rc_stored);
 #ifdef NETDATA_INTERNAL_CHECKS
-     else {
-         char host_uuid_str[UUID_STR_LEN];
-         uuid_unparse_lower(*host_uuid, host_uuid_str);
-         info("%s: Deleted context %s under host %s", __FUNCTION__ , context_data->id, host_uuid_str);
-     }
+    else {
+        char host_uuid_str[UUID_STR_LEN];
+        uuid_unparse_lower(*host_uuid, host_uuid_str);
+        info("%s: Deleted context %s under host %s", __FUNCTION__, context_data->id, host_uuid_str);
+    }
 #endif
 
 skip_delete:
@@ -449,6 +446,10 @@ skip_delete:
 int sql_context_cache_stats(int op)
 {
     int count, dummy;
+
+    if (unlikely(!db_context_meta))
+        return 0;
+
     netdata_thread_disable_cancelability();
     sqlite3_db_status(db_context_meta, op, &count, &dummy, 0);
     netdata_thread_enable_cancelability();
@@ -489,6 +490,8 @@ int ctx_unittest(void)
     uuid_t host_uuid;
     uuid_generate(host_uuid);
 
+    initialize_thread_key_pool();
+
     int rc = sql_init_context_database(1);
 
     if (rc != SQLITE_OK)
@@ -556,6 +559,7 @@ int ctx_unittest(void)
     freez((void *)context_data.title);
     freez((void *)context_data.chart_type);
     freez((void *)context_data.family);
+    freez((void *)context_data.units);
 
     // The list should be empty
     info("List context start after delete");
