@@ -14,7 +14,9 @@ void metadata_sync_shutdown_prepare(void);
 void metaqueue_delete_dimension_uuid(uuid_t *uuid);
 void metaqueue_store_claim_id(uuid_t *host_uuid, uuid_t *claim_uuid);
 void metaqueue_host_update_info(RRDHOST *host);
+void metaqueue_ml_load_models(RRDDIM *rd);
 void migrate_localhost(uuid_t *host_uuid);
+void metadata_queue_load_host_context(RRDHOST *host);
 
 // UNIT TEST
 int metadata_unittest(void);
