@@ -32,6 +32,7 @@
 #define HEALTH_REPEAT_KEY "repeat"
 #define HEALTH_HOST_LABEL_KEY "host labels"
 #define HEALTH_FOREACH_KEY "foreach"
+#define HEALTH_CHART_LABEL_KEY "chart labels"
 
 static inline int health_parse_delay(
         size_t line, const char *filename, char *string,
@@ -185,6 +186,95 @@ static inline int health_parse_repeat(
     return 1;
 }
 
+static inline int isvariableterm(const char s) {
+    if(isalnum(s) || s == '.' || s == '_')
+        return 0;
+
+    return 1;
+}
+
+// If needed, add a prefix key to all possible values in the range
+static inline char *health_config_add_key_to_values(char *value) {
+    BUFFER *wb = buffer_create(HEALTH_CONF_MAX_LINE + 1, NULL);
+    char key[HEALTH_CONF_MAX_LINE + 1];
+    char data[HEALTH_CONF_MAX_LINE + 1];
+
+    char *s = value;
+    size_t i = 0;
+
+    key[0] = '\0';
+    while(*s) {
+        if (*s == '=') {
+            //hold the key
+            data[i]='\0';
+            strncpyz(key, data, HEALTH_CONF_MAX_LINE);
+            i=0;
+        } else if (*s == ' ') {
+            data[i]='\0';
+            if (data[0]=='!')
+                buffer_snprintf(wb, HEALTH_CONF_MAX_LINE, "!%s=%s ", key, data + 1);
+            else
+                buffer_snprintf(wb, HEALTH_CONF_MAX_LINE, "%s=%s ", key, data);
+            i=0;
+        } else {
+            data[i++] = *s;
+        }
+        s++;
+    }
+
+    data[i]='\0';
+    if (data[0]) {
+        if (data[0]=='!')
+            buffer_snprintf(wb, HEALTH_CONF_MAX_LINE, "!%s=%s ", key, data + 1);
+        else
+            buffer_snprintf(wb, HEALTH_CONF_MAX_LINE, "%s=%s ", key, data);
+    }
+
+    char *final = mallocz(HEALTH_CONF_MAX_LINE + 1);
+    strncpyz(final, buffer_tostring(wb), HEALTH_CONF_MAX_LINE);
+    buffer_free(wb);
+
+    return final;
+}
+
+static inline void parse_variables_and_store_in_health_rrdvars(char *value, size_t len) {
+    const char *s = value;
+    char buffer[RRDVAR_MAX_LENGTH];
+
+    // $
+    while (*s) {
+        if(*s == '$') {
+            size_t i = 0;
+            s++;
+
+            if(*s == '{') {
+                // ${variable_name}
+
+                s++;
+                while (*s && *s != '}' && i < len)
+                    buffer[i++] = *s++;
+
+                if(*s == '}')
+                    s++;
+            }
+            else {
+                // $variable_name
+
+                while (*s && !isvariableterm(*s) && i < len)
+                    buffer[i++] = *s++;
+            }
+
+            buffer[i] = '\0';
+
+            //TODO: check and try to store only variables
+            STRING *name_string = rrdvar_name_to_string(buffer);
+            rrdvar_add("health", health_rrdvars, name_string, RRDVAR_TYPE_CALCULATED, RRDVAR_FLAG_CONFIG_VAR, NULL);
+            string_freez(name_string);
+        } else
+            s++;
+    }
+}
+
 /**
  * Health pattern from Foreach
  *
@@ -206,7 +296,7 @@ static SIMPLE_PATTERN *health_pattern_from_foreach(const char *s) {
 
     if(convert) {
         dimension_remove_pipe_comma(convert);
-        val = simple_pattern_create(convert, NULL, SIMPLE_PATTERN_EXACT);
+        val = simple_pattern_create(convert, NULL, SIMPLE_PATTERN_EXACT, true);
         freez(convert);
     }
 
@@ -215,7 +305,7 @@ static SIMPLE_PATTERN *health_pattern_from_foreach(const char *s) {
 
 static inline int health_parse_db_lookup(
         size_t line, const char *filename, char *string,
-        RRDR_GROUPING *group_method, int *after, int *before, int *every,
+        RRDR_TIME_GROUPING *group_method, int *after, int *before, int *every,
         RRDCALC_OPTIONS *options, STRING **dimensions, STRING **foreachdim
 ) {
     debug(D_HEALTH, "Health configuration parsing database lookup %zu@%s: %s", line, filename, string);
@@ -241,7 +331,7 @@ static inline int health_parse_db_lookup(
         return 0;
     }
 
-    if((*group_method = web_client_api_request_v1_data_group(key, RRDR_GROUPING_UNDEFINED)) == RRDR_GROUPING_UNDEFINED) {
+    if((*group_method = time_grouping_parse(key, RRDR_GROUPING_UNDEFINED)) == RRDR_GROUPING_UNDEFINED) {
         error("Health configuration at line %zu of file '%s': invalid group method '%s'",
                 line, filename, key);
         return 0;
@@ -408,6 +498,7 @@ static inline void alert_config_free(struct alert_config *cfg)
     string_freez(cfg->host_labels);
     string_freez(cfg->p_db_lookup_dimensions);
     string_freez(cfg->p_db_lookup_method);
+    string_freez(cfg->chart_labels);
     freez(cfg);
 }
 
@@ -444,7 +535,8 @@ static int health_readfile(const char *filename, void *data) {
             hash_delay = 0,
             hash_options = 0,
             hash_repeat = 0,
-            hash_host_label = 0;
+            hash_host_label = 0,
+            hash_chart_label = 0;
 
     char buffer[HEALTH_CONF_MAX_LINE + 1];
 
@@ -476,6 +568,7 @@ static int health_readfile(const char *filename, void *data) {
         hash_options = simple_uhash(HEALTH_OPTIONS_KEY);
         hash_repeat = simple_uhash(HEALTH_REPEAT_KEY);
         hash_host_label = simple_uhash(HEALTH_HOST_LABEL_KEY);
+        hash_chart_label = simple_uhash(HEALTH_CHART_LABEL_KEY);
     }
 
     FILE *fp = fopen(filename, "r");
@@ -634,9 +727,9 @@ static int health_readfile(const char *filename, void *data) {
         else if(hash == hash_os && !strcasecmp(key, HEALTH_OS_KEY)) {
             char *os_match = value;
             if (alert_cfg) alert_cfg->os = string_strdupz(value);
-            SIMPLE_PATTERN *os_pattern = simple_pattern_create(os_match, NULL, SIMPLE_PATTERN_EXACT);
+            SIMPLE_PATTERN *os_pattern = simple_pattern_create(os_match, NULL, SIMPLE_PATTERN_EXACT, true);
 
-            if(!simple_pattern_matches(os_pattern, rrdhost_os(host))) {
+            if(!simple_pattern_matches_string(os_pattern, host->os)) {
                 if(rc)
                     debug(D_HEALTH, "HEALTH on '%s' ignoring alarm '%s' defined at %zu@%s: host O/S does not match '%s'", rrdhost_hostname(host), rrdcalc_name(rc), line, filename, os_match);
 
@@ -651,9 +744,9 @@ static int health_readfile(const char *filename, void *data) {
         else if(hash == hash_host && !strcasecmp(key, HEALTH_HOST_KEY)) {
             char *host_match = value;
             if (alert_cfg) alert_cfg->host = string_strdupz(value);
-            SIMPLE_PATTERN *host_pattern = simple_pattern_create(host_match, NULL, SIMPLE_PATTERN_EXACT);
+            SIMPLE_PATTERN *host_pattern = simple_pattern_create(host_match, NULL, SIMPLE_PATTERN_EXACT, true);
 
-            if(!simple_pattern_matches(host_pattern, rrdhost_hostname(host))) {
+            if(!simple_pattern_matches_string(host_pattern, host->hostname)) {
                 if(rc)
                     debug(D_HEALTH, "HEALTH on '%s' ignoring alarm '%s' defined at %zu@%s: hostname does not match '%s'", rrdhost_hostname(host), rrdcalc_name(rc), line, filename, host_match);
 
@@ -728,7 +821,7 @@ static int health_readfile(const char *filename, void *data) {
                     if (rc->dimensions)
                         alert_cfg->p_db_lookup_dimensions = string_dup(rc->dimensions);
                     if (rc->group)
-                        alert_cfg->p_db_lookup_method = string_strdupz(group_method2string(rc->group));
+                        alert_cfg->p_db_lookup_method = string_strdupz(time_grouping_method2string(rc->group));
                     alert_cfg->p_db_lookup_options = rc->options;
                     alert_cfg->p_db_lookup_after = rc->after;
                     alert_cfg->p_db_lookup_before = rc->before;
@@ -769,6 +862,7 @@ static int health_readfile(const char *filename, void *data) {
                     error("Health configuration at line %zu of file '%s' for alarm '%s' at key '%s' has unparse-able expression '%s': %s at '%s'",
                             line, filename, rrdcalc_name(rc), key, value, expression_strerror(error), failed_at);
                 }
+                parse_variables_and_store_in_health_rrdvars(value, HEALTH_CONF_MAX_LINE);
             }
             else if(hash == hash_warn && !strcasecmp(key, HEALTH_WARN_KEY)) {
                 alert_cfg->warn = string_strdupz(value);
@@ -779,6 +873,7 @@ static int health_readfile(const char *filename, void *data) {
                     error("Health configuration at line %zu of file '%s' for alarm '%s' at key '%s' has unparse-able expression '%s': %s at '%s'",
                             line, filename, rrdcalc_name(rc), key, value, expression_strerror(error), failed_at);
                 }
+                parse_variables_and_store_in_health_rrdvars(value, HEALTH_CONF_MAX_LINE);
             }
             else if(hash == hash_crit && !strcasecmp(key, HEALTH_CRIT_KEY)) {
                 alert_cfg->crit = string_strdupz(value);
@@ -789,6 +884,7 @@ static int health_readfile(const char *filename, void *data) {
                     error("Health configuration at line %zu of file '%s' for alarm '%s' at key '%s' has unparse-able expression '%s': %s at '%s'",
                             line, filename, rrdcalc_name(rc), key, value, expression_strerror(error), failed_at);
                 }
+                parse_variables_and_store_in_health_rrdvars(value, HEALTH_CONF_MAX_LINE);
             }
             else if(hash == hash_exec && !strcasecmp(key, HEALTH_EXEC_KEY)) {
                 alert_cfg->exec = string_strdupz(value);
@@ -870,7 +966,8 @@ static int health_readfile(const char *filename, void *data) {
                     rc->host_labels = string_strdupz(tmp);
                     freez(tmp);
                 }
-                rc->host_labels_pattern = simple_pattern_create(rrdcalc_host_labels(rc), NULL, SIMPLE_PATTERN_EXACT);
+                rc->host_labels_pattern = simple_pattern_create(rrdcalc_host_labels(rc), NULL, SIMPLE_PATTERN_EXACT,
+                                                                true);
             }
             else if(hash == hash_plugin && !strcasecmp(key, HEALTH_PLUGIN_KEY)) {
                 alert_cfg->plugin = string_strdupz(value);
@@ -878,7 +975,7 @@ static int health_readfile(const char *filename, void *data) {
                 simple_pattern_free(rc->plugin_pattern);
 
                 rc->plugin_match = string_strdupz(value);
-                rc->plugin_pattern = simple_pattern_create(rrdcalc_plugin_match(rc), NULL, SIMPLE_PATTERN_EXACT);
+                rc->plugin_pattern = simple_pattern_create(rrdcalc_plugin_match(rc), NULL, SIMPLE_PATTERN_EXACT, true);
             }
             else if(hash == hash_module && !strcasecmp(key, HEALTH_MODULE_KEY)) {
                 alert_cfg->module = string_strdupz(value);
@@ -886,7 +983,28 @@ static int health_readfile(const char *filename, void *data) {
                 simple_pattern_free(rc->module_pattern);
 
                 rc->module_match = string_strdupz(value);
-                rc->module_pattern = simple_pattern_create(rrdcalc_module_match(rc), NULL, SIMPLE_PATTERN_EXACT);
+                rc->module_pattern = simple_pattern_create(rrdcalc_module_match(rc), NULL, SIMPLE_PATTERN_EXACT, true);
+            }
+            else if(hash == hash_chart_label && !strcasecmp(key, HEALTH_CHART_LABEL_KEY)) {
+                alert_cfg->chart_labels = string_strdupz(value);
+                if(rc->chart_labels) {
+                    if(strcmp(rrdcalc_chart_labels(rc), value) != 0)
+                        error("Health configuration at line %zu of file '%s' for alarm '%s' has key '%s' twice, once with value '%s' and later with value '%s'.",
+                              line, filename, rrdcalc_name(rc), key, value, value);
+
+                    string_freez(rc->chart_labels);
+                    simple_pattern_free(rc->chart_labels_pattern);
+                }
+
+                {
+                    char *tmp = simple_pattern_trim_around_equal(value);
+                    char *tmp_2 = health_config_add_key_to_values(tmp);
+                    rc->chart_labels = string_strdupz(tmp_2);
+                    freez(tmp);
+                    freez(tmp_2);
+                }
+                rc->chart_labels_pattern = simple_pattern_create(rrdcalc_chart_labels(rc), NULL, SIMPLE_PATTERN_EXACT,
+                                                                true);
             }
             else {
                 error("Health configuration at line %zu of file '%s' for alarm '%s' has unknown key '%s'.",
@@ -950,7 +1068,8 @@ static int health_readfile(const char *filename, void *data) {
                 simple_pattern_free(rt->family_pattern);
 
                 rt->family_match = string_strdupz(value);
-                rt->family_pattern = simple_pattern_create(rrdcalctemplate_family_match(rt), NULL, SIMPLE_PATTERN_EXACT);
+                rt->family_pattern = simple_pattern_create(rrdcalctemplate_family_match(rt), NULL, SIMPLE_PATTERN_EXACT,
+                                                           true);
             }
             else if(hash == hash_plugin && !strcasecmp(key, HEALTH_PLUGIN_KEY)) {
                 alert_cfg->plugin = string_strdupz(value);
@@ -958,7 +1077,8 @@ static int health_readfile(const char *filename, void *data) {
                 simple_pattern_free(rt->plugin_pattern);
 
                 rt->plugin_match = string_strdupz(value);
-                rt->plugin_pattern = simple_pattern_create(rrdcalctemplate_plugin_match(rt), NULL, SIMPLE_PATTERN_EXACT);
+                rt->plugin_pattern = simple_pattern_create(rrdcalctemplate_plugin_match(rt), NULL, SIMPLE_PATTERN_EXACT,
+                                                           true);
             }
             else if(hash == hash_module && !strcasecmp(key, HEALTH_MODULE_KEY)) {
                 alert_cfg->module = string_strdupz(value);
@@ -966,7 +1086,8 @@ static int health_readfile(const char *filename, void *data) {
                 simple_pattern_free(rt->module_pattern);
 
                 rt->module_match = string_strdupz(value);
-                rt->module_pattern = simple_pattern_create(rrdcalctemplate_module_match(rt), NULL, SIMPLE_PATTERN_EXACT);
+                rt->module_pattern = simple_pattern_create(rrdcalctemplate_module_match(rt), NULL, SIMPLE_PATTERN_EXACT,
+                                                           true);
             }
             else if(hash == hash_charts && !strcasecmp(key, HEALTH_CHARTS_KEY)) {
                 alert_cfg->charts = string_strdupz(value);
@@ -974,7 +1095,8 @@ static int health_readfile(const char *filename, void *data) {
                 simple_pattern_free(rt->charts_pattern);
 
                 rt->charts_match = string_strdupz(value);
-                rt->charts_pattern = simple_pattern_create(rrdcalctemplate_charts_match(rt), NULL, SIMPLE_PATTERN_EXACT);
+                rt->charts_pattern = simple_pattern_create(rrdcalctemplate_charts_match(rt), NULL, SIMPLE_PATTERN_EXACT,
+                                                           true);
             }
             else if(hash == hash_lookup && !strcasecmp(key, HEALTH_LOOKUP_KEY)) {
                 alert_cfg->lookup = string_strdupz(value);
@@ -989,7 +1111,7 @@ static int health_readfile(const char *filename, void *data) {
                         alert_cfg->p_db_lookup_dimensions = string_dup(rt->dimensions);
 
                     if (rt->group)
-                        alert_cfg->p_db_lookup_method = string_strdupz(group_method2string(rt->group));
+                        alert_cfg->p_db_lookup_method = string_strdupz(time_grouping_method2string(rt->group));
 
                     alert_cfg->p_db_lookup_options = rt->options;
                     alert_cfg->p_db_lookup_after = rt->after;
@@ -1031,6 +1153,7 @@ static int health_readfile(const char *filename, void *data) {
                     error("Health configuration at line %zu of file '%s' for template '%s' at key '%s' has unparse-able expression '%s': %s at '%s'",
                             line, filename, rrdcalctemplate_name(rt), key, value, expression_strerror(error), failed_at);
                 }
+                parse_variables_and_store_in_health_rrdvars(value, HEALTH_CONF_MAX_LINE);
             }
             else if(hash == hash_warn && !strcasecmp(key, HEALTH_WARN_KEY)) {
                 alert_cfg->warn = string_strdupz(value);
@@ -1041,6 +1164,7 @@ static int health_readfile(const char *filename, void *data) {
                     error("Health configuration at line %zu of file '%s' for template '%s' at key '%s' has unparse-able expression '%s': %s at '%s'",
                             line, filename, rrdcalctemplate_name(rt), key, value, expression_strerror(error), failed_at);
                 }
+                parse_variables_and_store_in_health_rrdvars(value, HEALTH_CONF_MAX_LINE);
             }
             else if(hash == hash_crit && !strcasecmp(key, HEALTH_CRIT_KEY)) {
                 alert_cfg->crit = string_strdupz(value);
@@ -1051,6 +1175,7 @@ static int health_readfile(const char *filename, void *data) {
                     error("Health configuration at line %zu of file '%s' for template '%s' at key '%s' has unparse-able expression '%s': %s at '%s'",
                             line, filename, rrdcalctemplate_name(rt), key, value, expression_strerror(error), failed_at);
                 }
+                parse_variables_and_store_in_health_rrdvars(value, HEALTH_CONF_MAX_LINE);
             }
             else if(hash == hash_exec && !strcasecmp(key, HEALTH_EXEC_KEY)) {
                 alert_cfg->exec = string_strdupz(value);
@@ -1130,7 +1255,30 @@ static int health_readfile(const char *filename, void *data) {
                     rt->host_labels = string_strdupz(tmp);
                     freez(tmp);
                 }
-                rt->host_labels_pattern = simple_pattern_create(rrdcalctemplate_host_labels(rt), NULL, SIMPLE_PATTERN_EXACT);
+
+                rt->host_labels_pattern = simple_pattern_create(rrdcalctemplate_host_labels(rt), NULL,
+                                                                SIMPLE_PATTERN_EXACT, true);
+            }
+            else if(hash == hash_chart_label && !strcasecmp(key, HEALTH_CHART_LABEL_KEY)) {
+                alert_cfg->chart_labels = string_strdupz(value);
+                if(rt->chart_labels) {
+                    if(strcmp(rrdcalctemplate_chart_labels(rt), value) != 0)
+                        error("Health configuration at line %zu of file '%s' for template '%s' has key '%s' twice, once with value '%s' and later with value '%s'. Using ('%s').",
+                              line, filename, rrdcalctemplate_name(rt), key, rrdcalctemplate_chart_labels(rt), value, value);
+
+                    string_freez(rt->chart_labels);
+                    simple_pattern_free(rt->chart_labels_pattern);
+                }
+
+                {
+                    char *tmp = simple_pattern_trim_around_equal(value);
+                    char *tmp_2 = health_config_add_key_to_values(tmp);
+                    rt->chart_labels = string_strdupz(tmp_2);
+                    freez(tmp);
+                    freez(tmp_2);
+                }
+                rt->chart_labels_pattern = simple_pattern_create(rrdcalctemplate_chart_labels(rt), NULL,
+                                                                SIMPLE_PATTERN_EXACT, true);
             }
             else {
                 error("Health configuration at line %zu of file '%s' for template '%s' has unknown key '%s'.",
@@ -1185,6 +1333,9 @@ void health_readdir(RRDHOST *host, const char *user_path, const char *stock_path
         stock_path = user_path;
     }
 
+    if (!health_rrdvars)
+        health_rrdvars = health_rrdvariables_create();
+
     recursive_config_double_dir_load(user_path, stock_path, subpath, health_readfile, (void *) host, 0);
     log_health("[%s]: Read health configuration.", rrdhost_hostname(host));
     sql_store_hashes = 0;
