@@ -5,14 +5,7 @@
 // ----------------------------------------------------------------------------
 
 inline void health_alarm_log_save(RRDHOST *host, ALARM_ENTRY *ae) {
-
     sql_health_alarm_log_save(host, ae);
-
-#ifdef ENABLE_ACLK
-    if (netdata_cloud_setting) {
-        sql_queue_alarm_to_aclk(host, ae, 0);
-    }
-#endif
 }
 
 // ----------------------------------------------------------------------------
@@ -53,6 +46,8 @@ inline ALARM_ENTRY* health_create_alarm_entry(
 
     uuid_copy(ae->config_hash_id, *((uuid_t *) config_hash_id));
 
+    uuid_generate_random(ae->transition_id);
+
     ae->family = string_dup(family);
     ae->classification = string_dup(class);
     ae->component = string_dup(component);
@@ -95,6 +90,8 @@ inline void health_alarm_log_add_entry(
 ) {
     debug(D_HEALTH, "Health adding alarm log entry with id: %u", ae->unique_id);
 
+    __atomic_add_fetch(&host->health_transitions, 1, __ATOMIC_RELAXED);
+
     // link it
     netdata_rwlock_wrlock(&host->health_log.alarm_log_rwlock);
     ae->next = host->health_log.alarms;
