@@ -1,7 +1,72 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
 
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE // for POLLRDHUP
+#endif
+
+#ifndef __BSD_VISIBLE
+#define __BSD_VISIBLE // for POLLRDHUP
+#endif
+
 #include "../libnetdata.h"
 
+
+SOCKET_PEERS socket_peers(int sock_fd) {
+    SOCKET_PEERS peers;
+
+    if(sock_fd < 0) {
+        strncpyz(peers.peer.ip, "unknown", sizeof(peers.peer.ip) - 1);
+        peers.peer.port = 0;
+
+        strncpyz(peers.local.ip, "unknown", sizeof(peers.local.ip) - 1);
+        peers.local.port = 0;
+
+        return peers;
+    }
+
+    struct sockaddr_storage addr;
+    socklen_t addr_len = sizeof(addr);
+
+    // Get peer info
+    if (getpeername(sock_fd, (struct sockaddr *)&addr, &addr_len) == 0) {
+        if (addr.ss_family == AF_INET) {  // IPv4
+            struct sockaddr_in *s = (struct sockaddr_in *)&addr;
+            inet_ntop(AF_INET, &s->sin_addr, peers.peer.ip, sizeof(peers.peer.ip));
+            peers.peer.port = ntohs(s->sin_port);
+        }
+        else {  // IPv6
+            struct sockaddr_in6 *s = (struct sockaddr_in6 *)&addr;
+            inet_ntop(AF_INET6, &s->sin6_addr, peers.peer.ip, sizeof(peers.peer.ip));
+            peers.peer.port = ntohs(s->sin6_port);
+        }
+    }
+    else {
+        strncpyz(peers.peer.ip, "unknown", sizeof(peers.peer.ip) - 1);
+        peers.peer.port = 0;
+    }
+
+    // Get local info
+    addr_len = sizeof(addr);
+    if (getsockname(sock_fd, (struct sockaddr *)&addr, &addr_len) == 0) {
+        if (addr.ss_family == AF_INET) {  // IPv4
+            struct sockaddr_in *s = (struct sockaddr_in *) &addr;
+            inet_ntop(AF_INET, &s->sin_addr, peers.local.ip, sizeof(peers.local.ip));
+            peers.local.port = ntohs(s->sin_port);
+        } else {  // IPv6
+            struct sockaddr_in6 *s = (struct sockaddr_in6 *) &addr;
+            inet_ntop(AF_INET6, &s->sin6_addr, peers.local.ip, sizeof(peers.local.ip));
+            peers.local.port = ntohs(s->sin6_port);
+        }
+    }
+    else {
+        strncpyz(peers.local.ip, "unknown", sizeof(peers.local.ip) - 1);
+        peers.local.port = 0;
+    }
+
+    return peers;
+}
+
+
 // --------------------------------------------------------------------------------------------------------------------
 // various library calls
 
@@ -11,6 +76,46 @@
 #define LARGE_SOCK_SIZE 4096
 #endif
 
+bool fd_is_socket(int fd) {
+    int type;
+    socklen_t len = sizeof(type);
+    if (getsockopt(fd, SOL_SOCKET, SO_TYPE, &type, &len) == -1)
+        return false;
+
+    return true;
+}
+
+bool sock_has_output_error(int fd) {
+    if(fd < 0) {
+        //internal_error(true, "invalid socket %d", fd);
+        return false;
+    }
+
+//    if(!fd_is_socket(fd)) {
+//        //internal_error(true, "fd %d is not a socket", fd);
+//        return false;
+//    }
+
+    short int errors = POLLERR | POLLHUP | POLLNVAL;
+
+#ifdef POLLRDHUP
+    errors |= POLLRDHUP;
+#endif
+
+    struct pollfd pfd = {
+            .fd = fd,
+            .events = POLLOUT | errors,
+            .revents = 0,
+    };
+
+    if(poll(&pfd, 1, 0) == -1) {
+        //internal_error(true, "poll() failed");
+        return false;
+    }
+
+    return ((pfd.revents & errors) || !(pfd.revents & POLLOUT));
+}
+
 int sock_setnonblock(int fd) {
     int flags;
 
@@ -919,66 +1024,11 @@ int connect_to_one_of_urls(const char *destination, int default_port, struct tim
 }
 
 
-#ifdef ENABLE_HTTPS
-ssize_t netdata_ssl_read(SSL *ssl, void *buf, size_t num) {
-    error_limit_static_thread_var(erl, 1, 0);
-
-    int bytes, err, retries = 0;
-
-    //do {
-    bytes = SSL_read(ssl, buf, (int)num);
-    err = SSL_get_error(ssl, bytes);
-    retries++;
-        //} while (bytes <= 0 && err == SSL_ERROR_WANT_READ);
-
-    if(unlikely(bytes <= 0)) {
-        if (err == SSL_ERROR_WANT_WRITE || err == SSL_ERROR_WANT_READ) {
-            bytes = 0;
-        } else
-            error("SSL_write() returned %d bytes, SSL error %d", bytes, err);
-    }
-
-    if(retries > 1)
-        error_limit(&erl, "SSL_read() retried %d times", retries);
-
-    return bytes;
-}
-
-ssize_t netdata_ssl_write(SSL *ssl, const void *buf, size_t num) {
-    error_limit_static_thread_var(erl, 1, 0);
-
-    int bytes, err, retries = 0;
-    size_t total = 0;
-
-    //do {
-    bytes = SSL_write(ssl, (uint8_t *)buf + total, (int)(num - total));
-    err = SSL_get_error(ssl, bytes);
-    retries++;
-
-    if(bytes > 0)
-        total += bytes;
-
-    //} while ((bytes <= 0 && (err == SSL_ERROR_WANT_WRITE)) || (bytes > 0 && total < num));
-
-    if(unlikely(bytes <= 0)) {
-        if (err == SSL_ERROR_WANT_WRITE || err == SSL_ERROR_WANT_READ) {
-            bytes = 0;
-        } else
-            error("SSL_write() returned %d bytes, SSL error %d", bytes, err);
-    }
-
-    if(retries > 1)
-        error_limit(&erl, "SSL_write() retried %d times", retries);
-
-    return bytes;
-}
-#endif
-
 // --------------------------------------------------------------------------------------------------------------------
 // helpers to send/receive data in one call, in blocking mode, with a timeout
 
 #ifdef ENABLE_HTTPS
-ssize_t recv_timeout(struct netdata_ssl *ssl,int sockfd, void *buf, size_t len, int flags, int timeout) {
+ssize_t recv_timeout(NETDATA_SSL *ssl,int sockfd, void *buf, size_t len, int flags, int timeout) {
 #else
 ssize_t recv_timeout(int sockfd, void *buf, size_t len, int flags, int timeout) {
 #endif
@@ -1007,19 +1057,22 @@ ssize_t recv_timeout(int sockfd, void *buf, size_t len, int flags, int timeout)
             return 0;
         }
 
-        if(fd.events & POLLIN) break;
+        if(fd.revents & POLLIN)
+            break;
     }
 
 #ifdef ENABLE_HTTPS
-    if (ssl->conn && ssl->flags == NETDATA_SSL_HANDSHAKE_COMPLETE)
-        return netdata_ssl_read(ssl->conn, buf, len);
+    if (SSL_connection(ssl)) {
+        return netdata_ssl_read(ssl, buf, len);
+    }
 #endif
 
+    internal_error(true, "%s(): calling recv()", __FUNCTION__ );
     return recv(sockfd, buf, len, flags);
 }
 
 #ifdef ENABLE_HTTPS
-ssize_t send_timeout(struct netdata_ssl *ssl,int sockfd, void *buf, size_t len, int flags, int timeout) {
+ssize_t send_timeout(NETDATA_SSL *ssl,int sockfd, void *buf, size_t len, int flags, int timeout) {
 #else
 ssize_t send_timeout(int sockfd, void *buf, size_t len, int flags, int timeout) {
 #endif
@@ -1048,13 +1101,13 @@ ssize_t send_timeout(int sockfd, void *buf, size_t len, int flags, int timeout)
             return 0;
         }
 
-        if(fd.events & POLLOUT) break;
+        if(fd.revents & POLLOUT) break;
     }
 
 #ifdef ENABLE_HTTPS
     if(ssl->conn) {
-        if (ssl->flags == NETDATA_SSL_HANDSHAKE_COMPLETE) {
-            return netdata_ssl_write(ssl->conn, buf, len);
+        if (SSL_connection(ssl)) {
+            return netdata_ssl_write(ssl, buf, len);
         }
         else {
             error("cannot write to SSL connection - connection is not ready.");
@@ -1118,8 +1171,8 @@ int accept4(int sock, struct sockaddr *addr, socklen_t *addrlen, int flags) {
  *                        update the client_host if uninitialized - ensure the hostsize is the number
  *                        of *writable* bytes (i.e. be aware of the strdup used to compact the pollinfo).
  */
-extern int connection_allowed(int fd, char *client_ip, char *client_host, size_t hostsize, SIMPLE_PATTERN *access_list,
-                              const char *patname, int allow_dns)
+int connection_allowed(int fd, char *client_ip, char *client_host, size_t hostsize, SIMPLE_PATTERN *access_list,
+                       const char *patname, int allow_dns)
 {
     debug(D_LISTENER,"checking %s... (allow_dns=%d)", patname, allow_dns);
     if (!access_list)
