@@ -17,13 +17,9 @@ fi
 
 # These options control which packages we are going to install
 # They can be pre-set, but also can be controlled with command line options
-PACKAGES_NETDATA=${PACKAGES_NETDATA-0}
-PACKAGES_NETDATA_NODEJS=${PACKAGES_NETDATA_NODEJS-0}
+PACKAGES_NETDATA=${PACKAGES_NETDATA-1}
 PACKAGES_NETDATA_PYTHON=${PACKAGES_NETDATA_PYTHON-0}
-PACKAGES_NETDATA_PYTHON3=${PACKAGES_NETDATA_PYTHON3-0}
-PACKAGES_NETDATA_PYTHON_MYSQL=${PACKAGES_NETDATA_PYTHON_MYSQL-0}
-PACKAGES_NETDATA_PYTHON_POSTGRES=${PACKAGES_NETDATA_PYTHON_POSTGRES-0}
-PACKAGES_NETDATA_PYTHON_MONGO=${PACKAGES_NETDATA_PYTHON_MONGO-0}
+PACKAGES_NETDATA_PYTHON3=${PACKAGES_NETDATA_PYTHON3-1}
 PACKAGES_DEBUG=${PACKAGES_DEBUG-0}
 PACKAGES_IPRANGE=${PACKAGES_IPRANGE-0}
 PACKAGES_FIREHOL=${PACKAGES_FIREHOL-0}
@@ -31,8 +27,8 @@ PACKAGES_FIREQOS=${PACKAGES_FIREQOS-0}
 PACKAGES_UPDATE_IPSETS=${PACKAGES_UPDATE_IPSETS-0}
 PACKAGES_NETDATA_DEMO_SITE=${PACKAGES_NETDATA_DEMO_SITE-0}
 PACKAGES_NETDATA_SENSORS=${PACKAGES_NETDATA_SENSORS-0}
-PACKAGES_NETDATA_DATABASE=${PACKAGES_NETDATA_DATABASE-0}
-PACKAGES_NETDATA_EBPF=${PACKAGES_NETDATA_EBPF-0}
+PACKAGES_NETDATA_DATABASE=${PACKAGES_NETDATA_DATABASE-1}
+PACKAGES_NETDATA_EBPF=${PACKAGES_NETDATA_EBPF-1}
 
 # needed commands
 lsb_release=$(command -v lsb_release 2> /dev/null)
@@ -99,34 +95,20 @@ Supported installers (IN):
 Supported packages (you can append many of them):
 
     - netdata-all    all packages required to install netdata
-                     including mysql client, postgres client,
-                     node.js, python, sensors, etc
+                     including python, sensors, etc
 
     - netdata        minimum packages required to install netdata
-                     (no mysql client, no nodejs, includes python)
-
-    - nodejs         install nodejs
-                     (required for monitoring named and SNMP)
+                     (includes python)
 
     - python         install python
 
     - python3        install python3
 
-    - python-mysql   install MySQLdb
-                     (for monitoring mysql, will install python3 version
-                     if python3 is enabled or detected)
-
-    - python-postgres install psycopg2
-                     (for monitoring postgres, will install python3 version
-                     if python3 is enabled or detected)
-
-    - python-pymongo install python-pymongo (or python3-pymongo for python3)
-
     - sensors        install lm_sensors for monitoring h/w sensors
 
-    - firehol-all    packages required for FireHOL, FireQoS, update-ipsets
+    - firehol-all    packages required for FireHOL, FireQOS, update-ipsets
     - firehol        packages required for FireHOL
-    - fireqos        packages required for FireQoS
+    - fireqos        packages required for FireQOS
     - update-ipsets  packages required for update-ipsets
 
     - demo           packages required for running a netdata demo site
@@ -198,7 +180,7 @@ get_os_release() {
   eval "$(grep -E "^(NAME|ID|ID_LIKE|VERSION|VERSION_ID)=" "${os_release_file}")"
   for x in "${ID}" ${ID_LIKE}; do
     case "${x,,}" in
-      alpine | arch | centos | clear-linux-os | debian | fedora | gentoo | manjaro | opensuse-leap | rhel | sabayon | sles | suse | ubuntu)
+      almalinux | alpine | arch | centos | clear-linux-os | debian | fedora | gentoo | manjaro | opensuse-leap | ol | rhel | rocky | sabayon | sles | suse | ubuntu)
         distribution="${x}"
         version="${VERSION_ID}"
         codename="${VERSION}"
@@ -345,7 +327,7 @@ user_picks_distribution() {
     if [ "${REPLY}" = "yum" ] && [ -z "${distribution}" ]; then
       REPLY=
       while [ -z "${REPLY}" ]; do
-        if ! read -r -p "yum in centos, rhel or fedora? > "; then
+        if ! read -r -p "yum in centos, rhel, ol or fedora? > "; then
           continue
         fi
 
@@ -353,11 +335,14 @@ user_picks_distribution() {
           fedora | rhel)
             distribution="rhel"
             ;;
+          ol)
+            distribution="ol"
+            ;;
           centos)
             distribution="centos"
             ;;
           *)
-            echo >&2 "Please enter 'centos', 'fedora' or 'rhel'."
+            echo >&2 "Please enter 'centos', 'fedora', 'ol' or 'rhel'."
             REPLY=
             ;;
         esac
@@ -416,13 +401,13 @@ detect_package_manager_from_distribution() {
       fi
       ;;
 
-    centos* | clearos*)
-      echo >&2 "You should have EPEL enabled to install all the prerequisites."
-      echo >&2 "Check: http://www.tecmint.com/how-to-enable-epel-repository-for-rhel-centos-6-5/"
-      package_installer="install_yum"
+    centos* | clearos* | rocky* | almalinux*)
+      package_installer=""
       tree="centos"
-      if [ "${IGNORE_INSTALLED}" -eq 0 ] && [ -z "${yum}" ]; then
-        echo >&2 "command 'yum' is required to install packages on a '${distribution} ${version}' system."
+      [ -n "${yum}" ] && package_installer="install_yum"
+      [ -n "${dnf}" ] && package_installer="install_dnf"
+      if [ "${IGNORE_INSTALLED}" -eq 0 ] && [ -z "${package_installer}" ]; then
+        echo >&2 "command 'yum' or 'dnf' is required to install packages on a '${distribution} ${version}' system."
         exit 1
       fi
       ;;
@@ -438,6 +423,17 @@ detect_package_manager_from_distribution() {
       fi
       ;;
 
+    ol*)
+      package_installer=
+      tree="ol"
+      [ -n "${yum}" ] && package_installer="install_yum"
+      [ -n "${dnf}" ] && package_installer="install_dnf"
+      if [ "${IGNORE_INSTALLED}" -eq 0 ] && [ -z "${package_installer}" ]; then
+        echo >&2 "command 'yum' or 'dnf' is required to install packages on a '${distribution} ${version}' system."
+        exit 1
+      fi
+      ;;
+
     suse* | opensuse* | sles*)
       package_installer="install_zypper"
       tree="suse"
@@ -499,7 +495,13 @@ check_package_manager() {
     dnf)
       [ "${IGNORE_INSTALLED}" -eq 0 ] && [ -z "${dnf}" ] && echo >&2 "${1} is not available." && return 1
       package_installer="install_dnf"
-      tree="rhel"
+      if [ "${distribution}" = "centos" ]; then
+        tree="centos"
+      elif [ "${distribution}" = "ol" ]; then
+        tree="ol"
+      else
+        tree="rhel"
+      fi
       detection="user-input"
       return 0
       ;;
@@ -550,6 +552,8 @@ check_package_manager() {
       package_installer="install_yum"
       if [ "${distribution}" = "centos" ]; then
         tree="centos"
+      elif [ "${distribution}" = "ol" ]; then
+        tree="ol"
       else
         tree="rhel"
       fi
@@ -642,6 +646,8 @@ declare -A pkg_autogen=(
   # exceptions
   ['centos-6']="WARNING|"
   ['rhel-6']="WARNING|"
+  ['centos-9']="NOTREQUIRED|"
+  ['rhel-9']="NOTREQUIRED|"
 )
 
 declare -A pkg_automake=(
@@ -650,13 +656,6 @@ declare -A pkg_automake=(
   ['default']="automake"
 )
 
-# required to bundle libJudy
-declare -A pkg_libtool=(
-  ['gentoo']="sys-devel/libtool"
-  ['clearlinux']="c-basic"
-  ['default']="libtool"
-)
-
 # Required to build libwebsockets and libmosquitto on some systems.
 declare -A pkg_cmake=(
   ['gentoo']="dev-util/cmake"
@@ -673,9 +672,37 @@ declare -A pkg_json_c_dev=(
   ['sabayon']="dev-libs/json-c"
   ['suse']="libjson-c-devel"
   ['freebsd']="json-c"
+  ['macos']="json-c"
   ['default']="json-c-devel"
 )
 
+#TODO:: clearlinux ?
+declare -A pkg_libyaml_dev=(
+  ['alpine']="yaml-dev"
+  ['arch']="libyaml"
+  ['clearlinux']="yaml-dev"
+  ['debian']="libyaml-dev"
+  ['gentoo']="dev-libs/libyaml"
+  ['sabayon']="dev-libs/libyaml"
+  ['suse']="libyaml-devel"
+  ['freebsd']="libyaml"
+  ['macos']="libyaml"
+  ['default']="libyaml-devel"
+)
+
+declare -A pkg_libatomic=(
+  ['arch']="NOTREQUIRED"
+  ['clearlinux']="NOTREQUIRED"
+  ['debian']="libatomic1"
+  ['freebsd']="NOTREQUIRED"
+  ['gentoo']="NOTREQUIRED"
+  ['macos']="NOTREQUIRED"
+  ['sabayon']="NOTREQUIRED"
+  ['suse']="libatomic1"
+  ['ubuntu']="libatomic1"
+  ['default']="libatomic"
+)
+
 declare -A pkg_bridge_utils=(
   ['gentoo']="net-misc/bridge-utils"
   ['clearlinux']="network-basic"
@@ -683,13 +710,6 @@ declare -A pkg_bridge_utils=(
   ['default']="bridge-utils"
 )
 
-declare -A pkg_chrony=(
-  ['gentoo']="net-misc/chrony"
-  ['clearlinux']="time-server-basic"
-  ['macos']="WARNING|"
-  ['default']="chrony"
-)
-
 declare -A pkg_curl=(
   ['gentoo']="net-misc/curl"
   ['sabayon']="net-misc/curl"
@@ -706,6 +726,7 @@ declare -A pkg_tar=(
   ['gentoo']="app-arch/tar"
   ['clearlinux']="os-core-update"
   ['macos']="NOTREQUIRED"
+  ['freebsd']="NOTREQUIRED"
   ['default']="tar"
 )
 
@@ -721,6 +742,21 @@ declare -A pkg_gcc=(
   ['default']="gcc"
 )
 
+# g++, required for building protobuf
+# All three cases of this not being required are systems that implicitly
+# include g++ when installing gcc.
+declare -A pkg_gxx=(
+  ['alpine']="g++"
+  ['arch']="NOTREQUIRED"
+  ['clearlinux']="c-basic"
+  ['debian']="g++"
+  ['gentoo']="NOTREQUIRED"
+  ['macos']="NOTREQUIRED"
+  ['ubuntu']="g++"
+  ['freebsd']="NOTREQUIRED"
+  ['default']="gcc-c++"
+)
+
 declare -A pkg_gdb=(
   ['gentoo']="sys-devel/gdb"
   ['macos']="NOTREQUIRED"
@@ -772,6 +808,7 @@ declare -A pkg_libz_dev=(
   ['gentoo']="sys-libs/zlib"
   ['sabayon']="sys-libs/zlib"
   ['rhel']="zlib-devel"
+  ['ol']="zlib-devel"
   ['suse']="zlib-devel"
   ['clearlinux']="devpkg-zlib"
   ['macos']="NOTREQUIRED"
@@ -788,8 +825,9 @@ declare -A pkg_libuuid_dev=(
   ['gentoo']="sys-apps/util-linux"
   ['sabayon']="sys-apps/util-linux"
   ['rhel']="libuuid-devel"
+  ['ol']="libuuid-devel"
   ['suse']="libuuid-devel"
-  ['macos']="NOTREQUIRED"
+  ['macos']="ossp-uuid"
   ['freebsd']="e2fsprogs-libuuid"
   ['default']=""
 )
@@ -802,6 +840,7 @@ declare -A pkg_libmnl_dev=(
   ['gentoo']="net-libs/libmnl"
   ['sabayon']="net-libs/libmnl"
   ['rhel']="libmnl-devel"
+  ['ol']="libmnl-devel"
   ['suse']="libmnl-devel"
   ['clearlinux']="devpkg-libmnl"
   ['macos']="NOTREQUIRED"
@@ -851,44 +890,11 @@ declare -A pkg_make=(
   ['default']="make"
 )
 
-declare -A pkg_netcat=(
-  ['alpine']="netcat-openbsd"
-  ['arch']="netcat"
-  ['centos']="nmap-ncat"
-  ['debian']="netcat"
-  ['gentoo']="net-analyzer/netcat"
-  ['sabayon']="net-analyzer/gnu-netcat"
-  ['rhel']="nmap-ncat"
-  ['suse']="netcat-openbsd"
-  ['clearlinux']="sysadmin-basic"
-  ['arch']="gnu-netcat"
-  ['macos']="NOTREQUIRED"
-  ['default']="netcat"
-
-  # exceptions
-  ['centos-6']="nc"
-  ['rhel-6']="nc"
-)
-
 declare -A pkg_nginx=(
   ['gentoo']="www-servers/nginx"
   ['default']="nginx"
 )
 
-declare -A pkg_nodejs=(
-  ['gentoo']="net-libs/nodejs"
-  ['clearlinux']="nodejs-basic"
-  ['freebsd']="node"
-  ['default']="nodejs"
-
-  # exceptions
-  ['rhel-6']="WARNING|To install nodejs check: https://nodejs.org/en/download/package-manager/"
-  ['rhel-7']="WARNING|To install nodejs check: https://nodejs.org/en/download/package-manager/"
-  ['centos-6']="WARNING|To install nodejs check: https://nodejs.org/en/download/package-manager/"
-  ['debian-6']="WARNING|To install nodejs check: https://nodejs.org/en/download/package-manager/"
-  ['debian-7']="WARNING|To install nodejs check: https://nodejs.org/en/download/package-manager/"
-)
-
 declare -A pkg_postfix=(
   ['gentoo']="mail-mta/postfix"
   ['macos']="WARNING|"
@@ -903,6 +909,7 @@ declare -A pkg_pkg_config=(
   ['gentoo']="virtual/pkgconfig"
   ['sabayon']="virtual/pkgconfig"
   ['rhel']="pkgconfig"
+  ['ol']="pkgconfig"
   ['suse']="pkg-config"
   ['freebsd']="pkgconf"
   ['clearlinux']="c-basic"
@@ -920,86 +927,6 @@ declare -A pkg_python=(
   ['centos-8']="python2"
 )
 
-declare -A pkg_python_mysqldb=(
-  ['alpine']="py-mysqldb"
-  ['arch']="mysql-python"
-  ['centos']="MySQL-python"
-  ['debian']="python-mysqldb"
-  ['gentoo']="dev-python/mysqlclient"
-  ['sabayon']="dev-python/mysqlclient"
-  ['rhel']="MySQL-python"
-  ['suse']="python-PyMySQL"
-  ['clearlinux']="WARNING|"
-  ['default']="python-mysql"
-
-  # exceptions
-  ['fedora-24']="python2-mysql"
-)
-
-declare -A pkg_python3_mysqldb=(
-  ['alpine']="WARNING|"
-  ['arch']="WARNING|"
-  ['centos']="WARNING|"
-  ['debian']="python3-mysqldb"
-  ['gentoo']="dev-python/mysqlclient"
-  ['sabayon']="dev-python/mysqlclient"
-  ['rhel']="WARNING|"
-  ['suse']="WARNING|"
-  ['clearlinux']="WARNING|"
-  ['macos']="WARNING|"
-  ['default']="WARNING|"
-
-  # exceptions
-  ['debian-6']="WARNING|"
-  ['debian-7']="WARNING|"
-  ['debian-8']="WARNING|"
-  ['ubuntu-12.04']="WARNING|"
-  ['ubuntu-12.10']="WARNING|"
-  ['ubuntu-13.04']="WARNING|"
-  ['ubuntu-13.10']="WARNING|"
-  ['ubuntu-14.04']="WARNING|"
-  ['ubuntu-14.10']="WARNING|"
-  ['ubuntu-15.04']="WARNING|"
-  ['ubuntu-15.10']="WARNING|"
-  ['centos-7']="python36-mysql"
-  ['centos-8']="python38-mysql"
-  ['rhel-7']="python36-mysql"
-  ['rhel-8']="python38-mysql"
-)
-
-declare -A pkg_python_psycopg2=(
-  ['alpine']="py-psycopg2"
-  ['arch']="python2-psycopg2"
-  ['centos']="python-psycopg2"
-  ['debian']="python-psycopg2"
-  ['gentoo']="dev-python/psycopg"
-  ['sabayon']="dev-python/psycopg:2"
-  ['rhel']="python-psycopg2"
-  ['suse']="python-psycopg2"
-  ['clearlinux']="WARNING|"
-  ['macos']="WARNING|"
-  ['default']="python-psycopg2"
-)
-
-declare -A pkg_python3_psycopg2=(
-  ['alpine']="py3-psycopg2"
-  ['arch']="python-psycopg2"
-  ['centos']="WARNING|"
-  ['debian']="WARNING|"
-  ['gentoo']="dev-python/psycopg"
-  ['sabayon']="dev-python/psycopg:2"
-  ['rhel']="WARNING|"
-  ['suse']="WARNING|"
-  ['clearlinux']="WARNING|"
-  ['macos']="WARNING|"
-  ['default']="WARNING|"
-
-  ['centos-7']="python3-psycopg2"
-  ['centos-8']="python38-psycopg2"
-  ['rhel-7']="python3-psycopg2"
-  ['rhel-8']="python38-psycopg2"
-)
-
 declare -A pkg_python_pip=(
   ['alpine']="py-pip"
   ['gentoo']="dev-python/pip"
@@ -1019,38 +946,6 @@ declare -A pkg_python3_pip=(
   ['default']="python3-pip"
 )
 
-declare -A pkg_python_pymongo=(
-  ['alpine']="WARNING|"
-  ['arch']="python2-pymongo"
-  ['centos']="WARNING|"
-  ['debian']="python-pymongo"
-  ['gentoo']="dev-python/pymongo"
-  ['suse']="python-pymongo"
-  ['clearlinux']="WARNING|"
-  ['rhel']="WARNING|"
-  ['macos']="WARNING|"
-  ['default']="python-pymongo"
-)
-
-declare -A pkg_python3_pymongo=(
-  ['alpine']="WARNING|"
-  ['arch']="python-pymongo"
-  ['centos']="WARNING|"
-  ['debian']="python3-pymongo"
-  ['gentoo']="dev-python/pymongo"
-  ['suse']="python3-pymongo"
-  ['clearlinux']="WARNING|"
-  ['rhel']="WARNING|"
-  ['freebsd']="py37-pymongo"
-  ['macos']="WARNING|"
-  ['default']="python3-pymongo"
-
-  ['centos-7']="python36-pymongo"
-  ['centos-8']="python3-pymongo"
-  ['rhel-7']="python36-pymongo"
-  ['rhel-8']="python3-pymongo"
-)
-
 declare -A pkg_python_requests=(
   ['alpine']="py-requests"
   ['arch']="python2-requests"
@@ -1084,6 +979,7 @@ declare -A pkg_python3_requests=(
   ['centos-8']="python3-requests"
   ['rhel-7']="python36-requests"
   ['rhel-8']="python3-requests"
+  ['ol-8']="python3-requests"
 )
 
 declare -A pkg_lz4=(
@@ -1120,21 +1016,10 @@ declare -A pkg_openssl=(
   ['gentoo']="dev-libs/openssl"
   ['arch']="openssl"
   ['freebsd']="openssl"
-  ['macos']="openssl@1.1"
+  ['macos']="openssl"
   ['default']="openssl-devel"
 )
 
-declare -A pkg_judy=(
-  ['debian']="libjudy-dev"
-  ['ubuntu']="libjudy-dev"
-  ['suse']="judy-devel"
-  ['gentoo']="dev-libs/judy"
-  ['arch']="judy"
-  ['freebsd']="Judy"
-  ['fedora']="Judy-devel"
-  ['default']="NOTREQUIRED"
-)
-
 declare -A pkg_python3=(
   ['gentoo']="dev-lang/python"
   ['sabayon']="dev-lang/python:3.4"
@@ -1187,6 +1072,7 @@ declare -A pkg_valgrind=(
 declare -A pkg_ulogd=(
   ['centos']="WARNING|"
   ['rhel']="WARNING|"
+  ['ol']="WARNING|"
   ['clearlinux']="WARNING|"
   ['gentoo']="app-admin/ulogd"
   ['arch']="ulogd"
@@ -1216,6 +1102,7 @@ declare -A pkg_libelf=(
   ['fedora']="elfutils-libelf-devel"
   ['centos']="elfutils-libelf-devel"
   ['rhel']="elfutils-libelf-devel"
+  ['ol']="elfutils-libelf-devel"
   ['clearlinux']="devpkg-elfutils"
   ['suse']="libelf-devel"
   ['macos']="NOTREQUIRED"
@@ -1285,19 +1172,20 @@ packages() {
   # basic build environment
 
   suitable_package distro-sdk
+  suitable_package libatomic
 
   require_cmd git || suitable_package git
   require_cmd find || suitable_package find
 
-  require_cmd gcc ||
+  require_cmd gcc || require_cmd clang ||
     require_cmd gcc-multilib || suitable_package gcc
+  require_cmd g++ || require_cmd clang++ || suitable_package gxx
 
   require_cmd make || suitable_package make
   require_cmd autoconf || suitable_package autoconf
   suitable_package autoconf-archive
   require_cmd autogen || suitable_package autogen
   require_cmd automake || suitable_package automake
-  require_cmd libtoolize || suitable_package libtool
   require_cmd pkg-config || suitable_package pkg-config
   require_cmd cmake || suitable_package cmake
 
@@ -1322,7 +1210,6 @@ packages() {
     require_cmd tar || suitable_package tar
     require_cmd curl || suitable_package curl
     require_cmd gzip || suitable_package gzip
-    require_cmd nc || suitable_package netcat
   fi
 
   # -------------------------------------------------------------------------
@@ -1354,6 +1241,7 @@ packages() {
     suitable_package libuuid-dev
     suitable_package libmnl-dev
     suitable_package json-c-dev
+    suitable_package libyaml-dev
   fi
 
   # -------------------------------------------------------------------------
@@ -1369,7 +1257,6 @@ packages() {
     suitable_package libuv
     suitable_package lz4
     suitable_package openssl
-    suitable_package judy
   fi
 
   # -------------------------------------------------------------------------
@@ -1378,25 +1265,14 @@ packages() {
     suitable_package libelf
   fi
 
-  # -------------------------------------------------------------------------
-  # scripting interpreters for netdata plugins
-
-  if [ "${PACKAGES_NETDATA_NODEJS}" -ne 0 ]; then
-    require_cmd nodejs node js || suitable_package nodejs
-  fi
-
   # -------------------------------------------------------------------------
   # python2
 
   if [ "${PACKAGES_NETDATA_PYTHON}" -ne 0 ]; then
     require_cmd python || suitable_package python
 
-    [ "${PACKAGES_NETDATA_PYTHON_MONGO}" -ne 0 ] && suitable_package python-pymongo
     # suitable_package python-requests
     # suitable_package python-pip
-
-    [ "${PACKAGES_NETDATA_PYTHON_MYSQL}" -ne 0 ] && suitable_package python-mysqldb
-    [ "${PACKAGES_NETDATA_PYTHON_POSTGRES}" -ne 0 ] && suitable_package python-psycopg2
   fi
 
   # -------------------------------------------------------------------------
@@ -1405,12 +1281,8 @@ packages() {
   if [ "${PACKAGES_NETDATA_PYTHON3}" -ne 0 ]; then
     require_cmd python3 || suitable_package python3
 
-    [ "${PACKAGES_NETDATA_PYTHON_MONGO}" -ne 0 ] && suitable_package python3-pymongo
     # suitable_package python3-requests
     # suitable_package python3-pip
-
-    [ "${PACKAGES_NETDATA_PYTHON_MYSQL}" -ne 0 ] && suitable_package python3-mysqldb
-    [ "${PACKAGES_NETDATA_PYTHON_POSTGRES}" -ne 0 ] && suitable_package python3-psycopg2
   fi
 
   # -------------------------------------------------------------------------
@@ -1519,11 +1391,43 @@ validate_tree_freebsd() {
   echo >&2 " > Checking for gmake ..."
   if ! pkg query %n-%v | grep -q gmake; then
     if prompt "gmake is required to build on FreeBSD and is not installed. Shall I install it?"; then
+      # shellcheck disable=2086
       run ${sudo} pkg install ${opts} gmake
     fi
   fi
 }
 
+validate_tree_ol() {
+  local opts=
+  if [ "${NON_INTERACTIVE}" -eq 1 ]; then
+    echo >&2 "Running in non-interactive mode"
+    opts="-y"
+  fi
+
+  if [[ "${version}" =~ ^8(\..*)?$ ]]; then
+    echo " > Checking for CodeReady Builder ..."
+    if ! run ${sudo} dnf repolist | grep -q codeready; then
+      if prompt "CodeReady Builder not found, shall I install it?"; then
+        cat > /etc/yum.repos.d/ol8_codeready.repo <<-EOF
+	[ol8_codeready_builder]
+	name=Oracle Linux \$releasever CodeReady Builder (\$basearch)
+	baseurl=http://yum.oracle.com/repo/OracleLinux/OL8/codeready/builder/\$basearch
+	gpgkey=file:///etc/pki/rpm-gpg/RPM-GPG-KEY-oracle
+	gpgcheck=1
+	enabled=1
+	EOF
+      fi
+    fi
+  elif [[ "${version}" =~ ^9(\..*)?$ ]]; then
+    echo " > Checking for CodeReady Builder ..."
+    if ! run ${sudo} dnf repolist enabled | grep -q codeready; then
+      if prompt "CodeReady Builder not enabled, shall I enable it?"; then
+        run ${sudo} dnf config-manager --set-enabled ol9_codeready_builder
+      fi
+    fi
+  fi
+}
+
 validate_tree_centos() {
   local opts=
   if [ "${NON_INTERACTIVE}" -eq 1 ]; then
@@ -1533,43 +1437,59 @@ validate_tree_centos() {
 
   echo >&2 " > CentOS Version: ${version} ..."
 
-  echo >&2 " > Checking for epel ..."
-  if ! rpm -qa | grep epel > /dev/null; then
-    if prompt "epel not found, shall I install it?"; then
-      run ${sudo} yum ${opts} install epel-release
-    fi
-  fi
-
-  if [[ "${version}" =~ ^8(\..*)?$ ]]; then
+  if [[ "${version}" =~ ^9(\..*)?$ ]]; then
     echo >&2 " > Checking for config-manager ..."
-    if ! run yum ${sudo} config-manager; then
+    if ! run ${sudo} dnf config-manager --help; then
       if prompt "config-manager not found, shall I install it?"; then
+        # shellcheck disable=2086
+        run ${sudo} dnf ${opts} install 'dnf-command(config-manager)'
+      fi
+    fi
+
+    echo >&2 " > Checking for CRB ..."
+    # shellcheck disable=2086
+    if ! run dnf ${sudo} repolist | grep CRB; then
+      if prompt "CRB not found, shall I install it?"; then
+        # shellcheck disable=2086
+        run ${sudo} dnf ${opts} config-manager --set-enabled crb
+      fi
+    fi
+  elif [[ "${version}" =~ ^8(\..*)?$ ]]; then
+    echo >&2 " > Checking for config-manager ..."
+    if ! run ${sudo} yum config-manager --help; then
+      if prompt "config-manager not found, shall I install it?"; then
+        # shellcheck disable=2086
         run ${sudo} yum ${opts} install 'dnf-command(config-manager)'
       fi
     fi
 
     echo >&2 " > Checking for PowerTools ..."
+    # shellcheck disable=2086
     if ! run yum ${sudo} repolist | grep PowerTools; then
       if prompt "PowerTools not found, shall I install it?"; then
+        # shellcheck disable=2086
         run ${sudo} yum ${opts} config-manager --set-enabled powertools
       fi
     fi
 
-    echo >&2 " > Checking for Okay ..."
-    if ! rpm -qa | grep okay > /dev/null; then
-      if prompt "okay not found, shall I install it?"; then
-        run ${sudo} yum ${opts} install http://repo.okay.com.mx/centos/8/x86_64/release/okay-release-1-3.el8.noarch.rpm
+    echo >&2 " > Updating libarchive ..."
+    # shellcheck disable=2086
+    run ${sudo} yum ${opts} install libarchive
+
+  elif [[ "${version}" =~ ^7(\..*)?$ ]]; then
+    echo >&2 " > Checking for EPEL ..."
+    if ! rpm -qa | grep epel-release > /dev/null; then
+      if prompt "EPEL not found, shall I install it?"; then
+        # shellcheck disable=2086
+        run ${sudo} yum ${opts} install epel-release
       fi
     fi
-
-    echo >&2 " > Installing Judy-devel directly ..."
-    run ${sudo} yum ${opts} install http://mirror.centos.org/centos/8/PowerTools/x86_64/os/Packages/Judy-devel-1.0.5-18.module_el8.1.0+217+4d875839.x86_64.rpm
-
   elif [[ "${version}" =~ ^6\..*$ ]]; then
     echo >&2 " > Detected CentOS 6.x ..."
     echo >&2 " > Checking for Okay ..."
     if ! rpm -qa | grep okay > /dev/null; then
       if prompt "okay not found, shall I install it?"; then
+        # shellcheck disable=2086
         run ${sudo} yum ${opts} install http://repo.okay.com.mx/centos/6/x86_64/release/okay-release-1-3.el6.noarch.rpm
       fi
     fi
@@ -1601,7 +1521,7 @@ install_yum() {
   read -r -a yum_opts <<< "${opts}"
 
   # install the required packages
-  run ${sudo} yum "${yum_opts[@]}" install "${@}" # --enablerepo=epel-testing
+  run ${sudo} yum "${yum_opts[@]}" install "${@}"
 }
 
 # -----------------------------------------------------------------------------
@@ -1732,7 +1652,7 @@ install_equo() {
 PACMAN_DB_SYNCED=0
 validate_install_pacman() {
 
-  if [ ${PACMAN_DB_SYNCED} -eq 0 ]; then
+  if [ "${PACMAN_DB_SYNCED}" -eq 0 ]; then
     echo >&2 " > Running pacman -Sy to sync the database"
     local x
     x=$(pacman -Sy)
@@ -1803,6 +1723,7 @@ install_zypper() {
   fi
 
   local opts="--ignore-unknown"
+  local install_opts="--allow-downgrade"
   if [ "${NON_INTERACTIVE}" -eq 1 ]; then
     echo >&2 "Running in non-interactive mode"
     # http://unix.stackexchange.com/questions/82016/how-to-use-zypper-in-bash-scripts-for-someone-coming-from-apt-get
@@ -1810,9 +1731,8 @@ install_zypper() {
   fi
 
   read -r -a zypper_opts <<< "$opts"
-
   # install the required packages
-  run ${sudo} zypper "${zypper_opts[@]}" install "${@}"
+  run ${sudo} zypper "${zypper_opts[@]}" install "${install_opts}" "${@}"
 }
 
 # -----------------------------------------------------------------------------
@@ -1912,7 +1832,7 @@ EOF
 remote_log() {
   # log success or failure on our system
   # to help us solve installation issues
-  curl > /dev/null 2>&1 -Ss --max-time 3 "https://registry.my-netdata.io/log/installer?status=${1}&error=${2}&distribution=${distribution}&version=${version}&installer=${package_installer}&tree=${tree}&detection=${detection}&netdata=${PACKAGES_NETDATA}&nodejs=${PACKAGES_NETDATA_NODEJS}&python=${PACKAGES_NETDATA_PYTHON}&python3=${PACKAGES_NETDATA_PYTHON3}&mysql=${PACKAGES_NETDATA_PYTHON_MYSQL}&postgres=${PACKAGES_NETDATA_PYTHON_POSTGRES}&pymongo=${PACKAGES_NETDATA_PYTHON_MONGO}&sensors=${PACKAGES_NETDATA_SENSORS}&database=${PACKAGES_NETDATA_DATABASE}&ebpf=${PACKAGES_NETDATA_EBPF}&firehol=${PACKAGES_FIREHOL}&fireqos=${PACKAGES_FIREQOS}&iprange=${PACKAGES_IPRANGE}&update_ipsets=${PACKAGES_UPDATE_IPSETS}&demo=${PACKAGES_NETDATA_DEMO_SITE}"
+  curl > /dev/null 2>&1 -Ss --max-time 3 "https://registry.my-netdata.io/log/installer?status=${1}&error=${2}&distribution=${distribution}&version=${version}&installer=${package_installer}&tree=${tree}&detection=${detection}&netdata=${PACKAGES_NETDATA}&python=${PACKAGES_NETDATA_PYTHON}&python3=${PACKAGES_NETDATA_PYTHON3}&sensors=${PACKAGES_NETDATA_SENSORS}&database=${PACKAGES_NETDATA_DATABASE}&ebpf=${PACKAGES_NETDATA_EBPF}&firehol=${PACKAGES_FIREHOL}&fireqos=${PACKAGES_FIREQOS}&iprange=${PACKAGES_IPRANGE}&update_ipsets=${PACKAGES_UPDATE_IPSETS}&demo=${PACKAGES_NETDATA_DEMO_SITE}"
 }
 
 if [ -z "${1}" ]; then
@@ -1973,17 +1893,10 @@ while [ -n "${1}" ]; do
 
     netdata-all)
       PACKAGES_NETDATA=1
-      PACKAGES_NETDATA_NODEJS=1
       if [ "${pv}" -eq 2 ]; then
         PACKAGES_NETDATA_PYTHON=1
-        PACKAGES_NETDATA_PYTHON_MYSQL=1
-        PACKAGES_NETDATA_PYTHON_POSTGRES=1
-        PACKAGES_NETDATA_PYTHON_MONGO=1
       else
         PACKAGES_NETDATA_PYTHON3=1
-        PACKAGES_NETDATA_PYTHON3_MYSQL=1
-        PACKAGES_NETDATA_PYTHON3_POSTGRES=1
-        PACKAGES_NETDATA_PYTHON3_MONGO=1
       fi
       PACKAGES_NETDATA_SENSORS=1
       PACKAGES_NETDATA_DATABASE=1
@@ -2005,42 +1918,6 @@ while [ -n "${1}" ]; do
       PACKAGES_NETDATA_PYTHON3=1
       ;;
 
-    python-mysql | mysql-python | mysqldb | netdata-mysql)
-      if [ "${pv}" -eq 2 ]; then
-        PACKAGES_NETDATA_PYTHON=1
-        PACKAGES_NETDATA_PYTHON_MYSQL=1
-      else
-        PACKAGES_NETDATA_PYTHON3=1
-        PACKAGES_NETDATA_PYTHON3_MYSQL=1
-      fi
-      ;;
-
-    python-postgres | postgres-python | psycopg2 | netdata-postgres)
-      if [ "${pv}" -eq 2 ]; then
-        PACKAGES_NETDATA_PYTHON=1
-        PACKAGES_NETDATA_PYTHON_POSTGRES=1
-      else
-        PACKAGES_NETDATA_PYTHON3=1
-        PACKAGES_NETDATA_PYTHON3_POSTGRES=1
-      fi
-      ;;
-
-    python-pymongo)
-      if [ "${pv}" -eq 2 ]; then
-        PACKAGES_NETDATA_PYTHON=1
-        PACKAGES_NETDATA_PYTHON_MONGO=1
-      else
-        PACKAGES_NETDATA_PYTHON3=1
-        PACKAGES_NETDATA_PYTHON3_MONGO=1
-      fi
-      ;;
-
-    nodejs | netdata-nodejs)
-      PACKAGES_NETDATA=1
-      PACKAGES_NETDATA_NODEJS=1
-      PACKAGES_NETDATA_DATABASE=1
-      ;;
-
     sensors | netdata-sensors)
       PACKAGES_NETDATA=1
       PACKAGES_NETDATA_PYTHON3=1
@@ -2058,17 +1935,10 @@ while [ -n "${1}" ]; do
 
     demo | all)
       PACKAGES_NETDATA=1
-      PACKAGES_NETDATA_NODEJS=1
       if [ "${pv}" -eq 2 ]; then
         PACKAGES_NETDATA_PYTHON=1
-        PACKAGES_NETDATA_PYTHON_MYSQL=1
-        PACKAGES_NETDATA_PYTHON_POSTGRES=1
-        PACKAGES_NETDATA_PYTHON_MONGO=1
       else
         PACKAGES_NETDATA_PYTHON3=1
-        PACKAGES_NETDATA_PYTHON3_MYSQL=1
-        PACKAGES_NETDATA_PYTHON3_POSTGRES=1
-        PACKAGES_NETDATA_PYTHON3_MONGO=1
       fi
       PACKAGES_DEBUG=1
       PACKAGES_IPRANGE=1
