@@ -26,9 +26,8 @@ if [ -d /opt/netdata/etc/netdata.old ]; then
 fi
 
 STARTIT=1
-AUTOUPDATE=0
 REINSTALL_OPTIONS=""
-RELEASE_CHANNEL="nightly" # check .travis/create_artifacts.sh before modifying
+RELEASE_CHANNEL="nightly"
 
 while [ "${1}" ]; do
   case "${1}" in
@@ -36,10 +35,7 @@ while [ "${1}" ]; do
       STARTIT=0
       REINSTALL_OPTIONS="${REINSTALL_OPTIONS} ${1}"
       ;;
-    "--auto-update" | "-u")
-      AUTOUPDATE=1
-      REINSTALL_OPTIONS="${REINSTALL_OPTIONS} ${1}"
-      ;;
+    "--auto-update" | "-u") ;;
     "--stable-channel")
       RELEASE_CHANNEL="stable"
       REINSTALL_OPTIONS="${REINSTALL_OPTIONS} ${1}"
@@ -49,6 +45,7 @@ while [ "${1}" ]; do
       REINSTALL_OPTIONS="${REINSTALL_OPTIONS} ${1}"
       ;;
     "--disable-telemetry")
+      NETDATA_DISABLE_TELEMETRY=1
       REINSTALL_OPTIONS="${REINSTALL_OPTIONS} ${1}"
       ;;
 
@@ -57,49 +54,14 @@ while [ "${1}" ]; do
   shift 1
 done
 
-if [ ! "${DO_NOT_TRACK:-0}" -eq 0 ] || [ -n "$DO_NOT_TRACK" ]; then
+if [ ! "${DISABLE_TELEMETRY:-0}" -eq 0 ] ||
+  [ -n "$DISABLE_TELEMETRY" ] ||
+  [ ! "${DO_NOT_TRACK:-0}" -eq 0 ] ||
+  [ -n "$DO_NOT_TRACK" ]; then
+  NETDATA_DISABLE_TELEMETRY=1
   REINSTALL_OPTIONS="${REINSTALL_OPTIONS} --disable-telemetry"
 fi
 
-deleted_stock_configs=0
-if [ ! -f "etc/netdata/.installer-cleanup-of-stock-configs-done" ]; then
-
-  # -----------------------------------------------------------------------------
-  progress "Deleting stock configuration files from user configuration directory"
-
-  declare -A configs_signatures=()
-  source "system/configs.signatures"
-
-  if [ ! -d etc/netdata ]; then
-    run mkdir -p etc/netdata
-  fi
-
-  md5sum="$(command -v md5sum 2> /dev/null || command -v md5 2> /dev/null)"
-  while IFS= read -r -d '' x; do
-    # find it relative filename
-    f="${x/etc\/netdata\//}"
-
-    # find the stock filename
-    t="${f/.conf.old/.conf}"
-    t="${t/.conf.orig/.conf}"
-
-    if [ -n "${md5sum}" ]; then
-      # find the checksum of the existing file
-      md5="$(${md5sum} < "${x}" | cut -d ' ' -f 1)"
-      #echo >&2 "md5: ${md5}"
-
-      # check if it matches
-      if [ "${configs_signatures[${md5}]}" = "${t}" ]; then
-        # it matches the default
-        run rm -f "${x}"
-        deleted_stock_configs=$((deleted_stock_configs + 1))
-      fi
-    fi
-  done < <(find etc -type f)
-
-  touch "etc/netdata/.installer-cleanup-of-stock-configs-done"
-fi
-
 # -----------------------------------------------------------------------------
 progress "Attempt to create user/group netdata/netadata"
 
@@ -120,6 +82,11 @@ if portable_add_group netdata; then
         run_failed "Failed to add netdata user to secondary groups"
       fi
     done
+    # Netdata must be able to read /etc/pve/qemu-server/* and /etc/pve/lxc/*
+    # for reading VMs/containers names, CPU and memory limits on Proxmox.
+    if [ -d "/etc/pve" ]; then
+      portable_add_user_to_group "www-data" netdata && NETDATA_ADDED_TO_GROUPS="${NETDATA_ADDED_TO_GROUPS} www-data"
+    fi
     NETDATA_USER="netdata"
     NETDATA_GROUP="netdata"
   else
@@ -138,7 +105,7 @@ install_netdata_logrotate || run_failed "Cannot install logrotate file for netda
 progress "Telemetry configuration"
 
 # Opt-out from telemetry program
-if [ -n "${NETDATA_DISABLE_TELEMETRY+x}" ]; then
+if [ -n "${NETDATA_DISABLE_TELEMETRY}" ]; then
   run touch "${NETDATA_USER_CONFIG_DIR}/.opt-out-from-anonymous-statistics"
 else
   printf "You can opt out from anonymous statistics via the --disable-telemetry option, or by creating an empty file %s \n\n" "${NETDATA_USER_CONFIG_DIR}/.opt-out-from-anonymous-statistics"
@@ -153,16 +120,8 @@ set_netdata_updater_channel || run_failed "Cannot set netdata updater tool relea
 
 # -----------------------------------------------------------------------------
 progress "Install (but not enable) netdata updater tool"
-cleanup_old_netdata_updater || run_failed "Cannot cleanup old netdata updater tool."
 install_netdata_updater || run_failed "Cannot install netdata updater tool."
 
-progress "Check if we must enable/disable the netdata updater tool"
-if [ "${AUTOUPDATE}" = "1" ]; then
-  enable_netdata_updater || run_failed "Cannot enable netdata updater tool"
-else
-  disable_netdata_updater || run_failed "Cannot disable netdata updater tool"
-fi
-
 # -----------------------------------------------------------------------------
 progress "creating quick links"
 
@@ -181,7 +140,7 @@ dir_should_be_link() {
   fi
 
   run ln -s "${t}" "${d}"
-  cd "${old}"
+  cd "${old}" || true
 }
 
 dir_should_be_link . bin sbin
@@ -198,35 +157,55 @@ dir_should_be_link . var/log/netdata netdata-logs
 
 dir_should_be_link etc/netdata ../../usr/lib/netdata/conf.d orig
 
-if [ ${deleted_stock_configs} -gt 0 ]; then
-  dir_should_be_link etc/netdata ../../usr/lib/netdata/conf.d "000.-.USE.THE.orig.LINK.TO.COPY.AND.EDIT.STOCK.CONFIG.FILES"
-fi
-
 # -----------------------------------------------------------------------------
 progress "fix permissions"
 
 run chmod g+rx,o+rx /opt
-run chown -R ${NETDATA_USER}:${NETDATA_GROUP} /opt/netdata
+run find /opt/netdata -type d -exec chmod go+rx '{}' \+
+run chown -R ${NETDATA_USER}:${NETDATA_GROUP} /opt/netdata/var
+
+if [ -d /opt/netdata/usr/libexec/netdata/plugins.d/ebpf.d ]; then
+  run chown -R root:${NETDATA_GROUP} /opt/netdata/usr/libexec/netdata/plugins.d/ebpf.d
+fi
 
 # -----------------------------------------------------------------------------
 
-progress "fix plugin permissions"
+progress "changing plugins ownership and permissions"
 
-for x in apps.plugin freeipmi.plugin ioping cgroup-network; do
+for x in apps.plugin perf.plugin slabinfo.plugin debugfs.plugin freeipmi.plugin ioping cgroup-network ebpf.plugin nfacct.plugin xenstat.plugin python.d.plugin charts.d.plugin go.d.plugin ioping.plugin cgroup-network-helper.sh; do
   f="usr/libexec/netdata/plugins.d/${x}"
-
   if [ -f "${f}" ]; then
     run chown root:${NETDATA_GROUP} "${f}"
-    run chmod 4750 "${f}"
   fi
 done
 
-# fix the fping binary
-if [ -f bin/fping ]; then
-  run chown root:${NETDATA_GROUP} bin/fping
-  run chmod 4750 bin/fping
+if command -v setcap >/dev/null 2>&1; then
+    run setcap "cap_dac_read_search,cap_sys_ptrace=ep" "usr/libexec/netdata/plugins.d/apps.plugin"
+    run setcap "cap_dac_read_search=ep" "usr/libexec/netdata/plugins.d/slabinfo.plugin"
+    run setcap "cap_dac_read_search=ep" "usr/libexec/netdata/plugins.d/debugfs.plugin"
+
+    if command -v capsh >/dev/null 2>&1 && capsh --supports=cap_perfmon 2>/dev/null ; then
+        run setcap "cap_perfmon=ep" "usr/libexec/netdata/plugins.d/perf.plugin"
+    else
+        run setcap "cap_sys_admin=ep" "usr/libexec/netdata/plugins.d/perf.plugin"
+    fi
+
+    run setcap "cap_net_admin,cap_net_raw=eip" "usr/libexec/netdata/plugins.d/go.d.plugin"
+else
+  for x in apps.plugin perf.plugin slabinfo.plugin debugfs.plugin; do
+    f="usr/libexec/netdata/plugins.d/${x}"
+    run chmod 4750 "${f}"
+  done
 fi
 
+for x in freeipmi.plugin ioping cgroup-network ebpf.plugin nfacct.plugin xenstat.plugin; do
+  f="usr/libexec/netdata/plugins.d/${x}"
+
+  if [ -f "${f}" ]; then
+    run chmod 4750 "${f}"
+  fi
+done
+
 # -----------------------------------------------------------------------------
 
 echo "Configure TLS certificate paths"
