@@ -137,55 +137,3 @@ void charts2json(RRDHOST *host, BUFFER *wb, int skip_volatile, int show_archived
 
     buffer_sprintf(wb, "\n\t]\n}\n");
 }
-
-// generate collectors list for the api/v1/info call
-
-struct collector {
-    const char *plugin;
-    const char *module;
-};
-
-struct array_printer {
-    int c;
-    BUFFER *wb;
-};
-
-static int print_collector_callback(const DICTIONARY_ITEM *item __maybe_unused, void *entry, void *data) {
-    struct array_printer *ap = (struct array_printer *)data;
-    BUFFER *wb = ap->wb;
-    struct collector *col=(struct collector *) entry;
-    if(ap->c) buffer_strcat(wb, ",");
-    buffer_strcat(wb, "\n\t\t{\n\t\t\t\"plugin\": \"");
-    buffer_strcat(wb, col->plugin);
-    buffer_strcat(wb, "\",\n\t\t\t\"module\": \"");
-    buffer_strcat(wb, col->module);
-    buffer_strcat(wb, "\"\n\t\t}");
-    (ap->c)++;
-    return 0;
-}
-
-void chartcollectors2json(RRDHOST *host, BUFFER *wb) {
-    DICTIONARY *dict = dictionary_create(DICT_OPTION_SINGLE_THREADED);
-    RRDSET *st;
-    char name[500];
-
-    time_t now = now_realtime_sec();
-    rrdset_foreach_read(st, host) {
-        if (rrdset_is_available_for_viewers(st)) {
-            struct collector col = {
-                    .plugin = rrdset_plugin_name(st),
-                    .module = rrdset_module_name(st)
-            };
-            sprintf(name, "%s:%s", col.plugin, col.module);
-            dictionary_set(dict, name, &col, sizeof(struct collector));
-            st->last_accessed_time_s = now;
-        }
-    }
-    rrdset_foreach_done(st);
-    struct array_printer ap = {
-            .c = 0,
-            .wb = wb
-    };
-    dictionary_walkthrough_read(dict, print_collector_callback, &ap);
-    dictionary_destroy(dict);
-}
